searchData={"items":[{"type":"module","title":"Req","doc":"The high-level API.\n\nReq is composed of:\n\n  * `Req` - the high-level API (you're here!)\n\n  * `Req.Request` - the low-level API and the request struct\n\n  * `Req.Steps` - the collection of built-in steps\n\n  * `Req.Test` - the testing conveniences\n\nThe high-level API is what most users of Req will use most of the time.","ref":"Req.html"},{"type":"module","title":"Examples - Req","doc":"Making a GET request with `Req.get!/1`:\n\n    iex> Req.get!(\"https://api.github.com/repos/wojtekmach/req\").body[\"description\"]\n    \"Req is a batteries-included HTTP client for Elixir.\"\n\nSame, but by explicitly building request struct first:\n\n    iex> req = Req.new(base_url: \"https://api.github.com\")\n    iex> Req.get!(req, url: \"/repos/wojtekmach/req\").body[\"description\"]\n    \"Req is a batteries-included HTTP client for Elixir.\"\n\nReturn the request that was sent using `Req.run!/2`:\n\n    iex> {req, resp} = Req.run!(\"https://httpbin.org/basic-auth/foo/bar\", auth: {:basic, \"foo:bar\"})\n    iex> req.headers[\"authorization\"]\n    [\"Basic Zm9vOmJhcg==\"]\n    iex> resp.status\n    200\n\nMaking a POST request with `Req.post!/2`:\n\n    iex> Req.post!(\"https://httpbin.org/post\", form: [comments: \"hello!\"]).body[\"form\"]\n    %{\"comments\" => \"hello!\"}\n\nSet connection timeout:\n\n    iex> resp = Req.get!(\"https://httpbin.org\", connect_options: [timeout: 100])\n    iex> resp.status\n    200\n\nSee [`run_finch`](`Req.Steps.run_finch/1`) for more connection related options and usage examples.\n\nStream request body:\n\n    iex> stream = Stream.duplicate(\"foo\", 3)\n    iex> Req.post!(\"https://httpbin.org/post\", body: stream).body[\"data\"]\n    \"foofoofoo\"\n\nStream response body using a callback:\n\n    iex> resp =\n    ...>   Req.get!(\"http://httpbin.org/stream/2\", into: fn {:data, data}, {req, resp} ->\n    ...>     IO.puts(data)\n    ...>     {:cont, {req, resp}}\n    ...>   end)\n    # output: {\"url\": \"http://httpbin.org/stream/2\", ...}\n    # output: {\"url\": \"http://httpbin.org/stream/2\", ...}\n    iex> resp.status\n    200\n    iex> resp.body\n    \"\"\n\nStream response body into a `Collectable`:\n\n    iex> resp = Req.get!(\"http://httpbin.org/stream/2\", into: IO.stream())\n    # output: {\"url\": \"http://httpbin.org/stream/2\", ...}\n    # output: {\"url\": \"http://httpbin.org/stream/2\", ...}\n    iex> resp.status\n    200\n    iex> resp.body\n    %IO.Stream{}\n\nStream response body to the current process and parse incoming messages using `Req.parse_message/2`.\n\n    iex> resp = Req.get!(\"http://httpbin.org/stream/2\", into: :self)\n    iex> Req.parse_message(resp, receive do message -> message end)\n    {:ok, [data: \"{\\\"url\\\": \\\"http://httpbin.org/stream/2\\\", ..., \\\"id\\\": 0}\\n\"]}\n    iex> Req.parse_message(resp, receive do message -> message end)\n    {:ok, [data: \"{\\\"url\\\": \\\"http://httpbin.org/stream/2\\\", ..., \\\"id\\\": 1}\\n\"]}\n    iex> Req.parse_message(resp, receive do message -> message end)\n    {:ok, [:done]}\n    \"\"\n\nSame as above, using enumerable API:\n\n    iex> resp = Req.get!(\"http://httpbin.org/stream/2\", into: :self)\n    iex> resp.body\n    #Req.Response.Async<...>\n    iex> Enum.each(resp.body, &IO.puts/1)\n    # {\"url\": \"http://httpbin.org/stream/2\", ..., \"id\": 0}\n    # {\"url\": \"http://httpbin.org/stream/2\", ..., \"id\": 1}\n    :ok\n\nSee `:into` option in `Req.new/1` documentation for more information on response body streaming.","ref":"Req.html#module-examples"},{"type":"module","title":"Header Names - Req","doc":"The HTTP specification requires that header names should be case-insensitive.\nReq allows two ways to access the headers; using functions and by accessing\nthe data directly:\n\n    iex> Req.Response.get_header(response, \"content-type\")\n    [\"text/html\"]\n\n    iex> response.headers[\"content-type\"]\n    [\"text/html\"]\n\nWhile we can ensure case-insensitive handling in the former case, we can't\nin the latter. For this reason, Req made the following design choices:\n\n  * header names are stored as downcased\n\n  * functions like `Req.Request.get_header/2`, `Req.Request.put_header/3`,\n    `Req.Response.get_header/2`, `Req.Response.put_header/3`, etc\n    automatically downcase the given header name.","ref":"Req.html#module-header-names"},{"type":"function","title":"Req.cancel_async_response/1","doc":"Cancels an asynchronous response.\n\nAn asynchronous response is a result of request with `into: :self`.\nSee also `Req.Response.Async`.","ref":"Req.html#cancel_async_response/1"},{"type":"function","title":"Examples - Req.cancel_async_response/1","doc":"iex> resp = Req.get!(\"http://httpbin.org/stream/2\", into: :self)\n    iex> Req.cancel_async_response(resp)\n    :ok","ref":"Req.html#cancel_async_response/1-examples"},{"type":"function","title":"Req.default_options/0","doc":"Returns default options.\n\nSee `default_options/1` for more information.","ref":"Req.html#default_options/0"},{"type":"function","title":"Req.default_options/1","doc":"Sets default options for `Req.new/1`.\n\nAvoid setting default options in libraries as they are global.","ref":"Req.html#default_options/1"},{"type":"function","title":"Examples - Req.default_options/1","doc":"iex> Req.default_options(base_url: \"https://httpbin.org\")\n    iex> Req.get!(\"/statuses/201\").status\n    201\n    iex> Req.new() |> Req.get!(url: \"/statuses/201\").status\n    201","ref":"Req.html#default_options/1-examples"},{"type":"function","title":"Req.delete/2","doc":"Makes a DELETE request and returns a response or an error.\n\n`request` can be one of:\n\n  * an url (`String` or `URI`);\n\n  * a `Keyword` options;\n\n  * a `Req.Request` struct\n\nSee `new/1` for a list of available options.","ref":"Req.html#delete/2"},{"type":"function","title":"Examples - Req.delete/2","doc":"With URL:\n\n    iex> {:ok, resp} = Req.delete(\"https://httpbin.org/anything\")\n    iex> resp.body[\"method\"]\n    \"DELETE\"\n\nWith options:\n\n    iex> {:ok, resp} = Req.delete(url: \"https://httpbin.org/anything\")\n    iex> resp.body[\"method\"]\n    \"DELETE\"\n\nWith request struct:\n\n    iex> req = Req.new(url: \"https://httpbin.org/anything\")\n    iex> {:ok, resp} = Req.delete(req)\n    iex> resp.body[\"method\"]\n    \"DELETE\"","ref":"Req.html#delete/2-examples"},{"type":"function","title":"Req.delete!/2","doc":"Makes a DELETE request and returns a response or raises an error.\n\n`request` can be one of:\n\n  * an url (`String` or `URI`);\n\n  * a `Keyword` options;\n\n  * a `Req.Request` struct\n\nSee `new/1` for a list of available options.","ref":"Req.html#delete!/2"},{"type":"function","title":"Examples - Req.delete!/2","doc":"With URL:\n\n    iex> Req.delete!(\"https://httpbin.org/anything\").body[\"method\"]\n    \"DELETE\"\n\nWith options:\n\n    iex> Req.delete!(url: \"https://httpbin.org/anything\").body[\"method\"]\n    \"DELETE\"\n\nWith request struct:\n\n    iex> req = Req.new(url: \"https://httpbin.org/anything\")\n    iex> Req.delete!(req).body[\"method\"]\n    \"DELETE\"","ref":"Req.html#delete!/2-examples"},{"type":"function","title":"Req.get/2","doc":"Makes a GET request and returns a response or an error.\n\n`request` can be one of:\n\n  * an url (`String` or `URI`);\n\n  * a `Keyword` options;\n\n  * a `Req.Request` struct\n\nSee `new/1` for a list of available options.","ref":"Req.html#get/2"},{"type":"function","title":"Examples - Req.get/2","doc":"With URL:\n\n    iex> {:ok, resp} = Req.get(\"https://api.github.com/repos/wojtekmach/req\")\n    iex> resp.body[\"description\"]\n    \"Req is a batteries-included HTTP client for Elixir.\"\n\nWith options:\n\n    iex> {:ok, resp} = Req.get(url: \"https://api.github.com/repos/wojtekmach/req\")\n    iex> resp.status\n    200\n\nWith request struct:\n\n    iex> req = Req.new(base_url: \"https://api.github.com\")\n    iex> {:ok, resp} = Req.get(req, url: \"/repos/elixir-lang/elixir\")\n    iex> resp.status\n    200","ref":"Req.html#get/2-examples"},{"type":"function","title":"Req.get!/2","doc":"Makes a GET request and returns a response or raises an error.\n\n`request` can be one of:\n\n  * an url (`String` or `URI`);\n\n  * a `Keyword` options;\n\n  * a `Req.Request` struct\n\nSee `new/1` for a list of available options.","ref":"Req.html#get!/2"},{"type":"function","title":"Examples - Req.get!/2","doc":"With URL:\n\n    iex> Req.get!(\"https://api.github.com/repos/wojtekmach/req\").body[\"description\"]\n    \"Req is a batteries-included HTTP client for Elixir.\"\n\nWith options:\n\n    iex> Req.get!(url: \"https://api.github.com/repos/wojtekmach/req\").status\n    200\n\nWith request struct:\n\n    iex> req = Req.new(base_url: \"https://api.github.com\")\n    iex> Req.get!(req, url: \"/repos/elixir-lang/elixir\").status\n    200","ref":"Req.html#get!/2-examples"},{"type":"function","title":"Req.head/2","doc":"Makes a HEAD request and returns a response or an error.\n\n`request` can be one of:\n\n  * an url (`String` or `URI`);\n\n  * a `Keyword` options;\n\n  * a `Req.Request` struct\n\nSee `new/1` for a list of available options.","ref":"Req.html#head/2"},{"type":"function","title":"Examples - Req.head/2","doc":"With URL:\n\n    iex> {:ok, resp} = Req.head(\"https://httpbin.org/status/201\")\n    iex> resp.status\n    201\n\nWith options:\n\n    iex> {:ok, resp} = Req.head(url: \"https://httpbin.org/status/201\")\n    iex> resp.status\n    201\n\nWith request struct:\n\n    iex> req = Req.new(base_url: \"https://httpbin.org\")\n    iex> {:ok, resp} = Req.head(req, url: \"/status/201\")\n    iex> resp.status\n    201","ref":"Req.html#head/2-examples"},{"type":"function","title":"Req.head!/2","doc":"Makes a HEAD request and returns a response or raises an error.\n\n`request` can be one of:\n\n  * an url (`String` or `URI`);\n\n  * a `Keyword` options;\n\n  * a `Req.Request` struct\n\nSee `new/1` for a list of available options.","ref":"Req.html#head!/2"},{"type":"function","title":"Examples - Req.head!/2","doc":"With URL:\n\n    iex> Req.head!(\"https://httpbin.org/status/201\").status\n    201\n\nWith options:\n\n    iex> Req.head!(url: \"https://httpbin.org/status/201\").status\n    201\n\nWith request struct:\n\n    iex> req = Req.new(base_url: \"https://httpbin.org\")\n    iex> Req.head!(req, url: \"/status/201\").status\n    201","ref":"Req.html#head!/2-examples"},{"type":"function","title":"Req.merge/2","doc":"Updates a request struct.\n\nSee `new/1` for a list of available options. Also see `Req.Request` module documentation\nfor more information on the underlying request struct.","ref":"Req.html#merge/2"},{"type":"function","title":"Examples - Req.merge/2","doc":"iex> req = Req.new(base_url: \"https://httpbin.org\")\n    iex> req = Req.merge(req, auth: {:basic, \"alice:secret\"})\n    iex> req.options[:base_url]\n    \"https://httpbin.org\"\n    iex> req.options[:auth]\n    {:basic, \"alice:secret\"}\n\nPassing `:headers` will automatically encode and merge them:\n\n    iex> req = Req.new(headers: %{point_x: 1})\n    iex> req = Req.merge(req, headers: %{point_y: 2})\n    iex> req.headers\n    %{\"point-x\" => [\"1\"], \"point-y\" => [\"2\"]}\n\nThe same header names are overwritten however:\n\n    iex> req = Req.new(headers: %{authorization: \"bearer foo\"})\n    iex> req = Req.merge(req, headers: %{authorization: \"bearer bar\"})\n    iex> req.headers\n    %{\"authorization\" => [\"bearer bar\"]}\n\nSimilarly to headers, `:params` are merged too:\n\n    req = Req.new(url: \"https://httpbin.org/anything\", params: [a: 1, b: 1])\n    req = Req.merge(req, params: [a: 2])\n    Req.get!(req).body[\"args\"]\n    #=> %{\"a\" => \"2\", \"b\" => \"1\"}","ref":"Req.html#merge/2-examples"},{"type":"function","title":"Req.new/1","doc":"Returns a new request struct with built-in steps.\n\nSee `request/2`, `run/2`, as well as `get/2`, `post/2`, and similar functions for\nmaking requests.\n\nAlso see `Req.Request` module documentation for more information on the underlying request\nstruct.","ref":"Req.html#new/1"},{"type":"function","title":"Options - Req.new/1","doc":"Basic request options:\n\n  * `:method` - the request method, defaults to `:get`.\n\n  * `:url` - the request URL.\n\n  * `:headers` - the request headers as a `{key, value}` enumerable (e.g. map, keyword list).\n\n    The header names should be downcased.\n\n    The headers are automatically encoded using these rules:\n\n      * atom header names are turned into strings, replacing `_` with `-`. For example,\n        `:user_agent` becomes `\"user-agent\"`.\n\n      * string header names are downcased.\n\n      * `%DateTime{}` header values are encoded as \"HTTP date\".\n\n    If you set `:headers` options both in `Req.new/1` and `request/2`, the header lists are merged.\n\n    See also \"Header Names\" section in the module documentation.\n\n  * `:body` - the request body.\n\n    Can be one of:\n\n      * `iodata` - send request body eagerly\n\n      * `enumerable` - stream `enumerable` as request body\n\nAdditional URL options:\n\n  * `:base_url` - if set, the request URL is prepended with this base URL (via\n    [`put_base_url`](`Req.Steps.put_base_url/1`) step.)\n\n  * `:params` - if set, appends parameters to the request query string (via\n    [`put_params`](`Req.Steps.put_params/1`) step.)\n\n  * `:path_params` - if set, uses a templated request path (via\n    [`put_path_params`](`Req.Steps.put_path_params/1`) step.)\n\n  * `:path_params_style` (*available since v0.5.1*) - how path params are expressed (via\n    [`put_path_params`](`Req.Steps.put_path_params/1`) step). Can be one of:\n\n       * `:colon` - (default) for Plug-style parameters, such as `:code` in\n         `https://httpbin.org/status/:code`.\n\n       * `:curly` - for [OpenAPI](https://swagger.io/specification/)-style parameters, such as\n         `{code}` in `https://httpbin.org/status/{code}`.\n\nAuthentication options:\n\n  * `:auth` - sets request authentication (via [`auth`](`Req.Steps.auth/1`) step.)\n\n    Can be one of:\n\n      * `{:basic, userinfo}` - uses Basic HTTP authentication.\n\n      * `{:bearer, token}` - uses Bearer HTTP authentication.\n\n      * `:netrc` - load credentials from the default .netrc file.\n\n      * `{:netrc, path}` - load credentials from `path`.\n\n      * `string` - sets to this value.\n\n      * `&fun/0` - a function that returns one of the above (such as a `{:bearer, token}`).\n\nRequest body encoding options ([`encode_body`](`Req.Steps.encode_body/1`)):\n\n  * `:form` - if set, encodes the request body as `application/x-www-form-urlencoded`\n\n  * `:form_multipart` - if set, encodes the request body as `multipart/form-data`.\n\n  * `:json` - if set, encodes the request body as JSON\n\nOther request body options:\n\n  * `:compress_body` - if set to `true`, compresses the request body using gzip (via [`compress_body`](`Req.Steps.compress_body/1`) step.)\n    Defaults to `false`.\n\nAWS Signature Version 4 options ([`put_aws_sigv4`](`Req.Steps.put_aws_sigv4/1`) step):\n\n  * `:aws_sigv4` - if set, the AWS options to sign request:\n\n      * `:access_key_id` - the AWS access key id.\n\n      * `:secret_access_key` - the AWS secret access key.\n\n      * `:service` - the AWS service.\n\n      * `:region` - if set, AWS region. Defaults to `\"us-east-1\"`.\n\n      * `:datetime` - the request datetime, defaults to `DateTime.utc_now(:second)`.\n\nResponse body options:\n\n  * `:compressed` - if set to `true`, asks the server to return compressed response.\n    (via [`compressed`](`Req.Steps.compressed/1`) step.) Defaults to `true`.\n\n  * `:raw` - if set to `true`, disables automatic body decompression\n    ([`decompress_body`](`Req.Steps.decompress_body/1`) step) and decoding\n    ([`decode_body`](`Req.Steps.decode_body/1`) step.) Defaults to `false`.\n\n  * `:decode_body` - if set to `false`, disables automatic response body decoding.\n    Defaults to `true`.\n\n  * `:decode_json` - options to pass to `Jason.decode!/2`, defaults to `[]`.\n\n  * `:into` - where to send the response body. It can be one of:\n\n      * `nil` - (default) read the whole response body and store it in the `response.body`\n        field.\n\n      * `fun` - stream response body using a function. The first argument is a `{:data, data}`\n        tuple containing the chunk of the response body. The second argument is a\n        `{request, response}` tuple. To continue streaming chunks, return `{:cont, {req, resp}}`.\n        To cancel, return `{:halt, {req, resp}}`. For example:\n\n            into: fn {:data, data}, {req, resp} ->\n              IO.puts(data)\n              {:cont, {req, resp}}\n            end\n\n      * `collectable` - stream response body into a `t:Collectable.t/0`. For example:\n\n             into: File.stream!(\"path\")\n\n        Note that the collectable is only used, if the response status is 200. In other cases,\n        the body is accumulated and processed as usual.\n\n      * `:self` - stream response body into the current process mailbox.\n\n        Received messages should be parsed with `Req.parse_message/2`.\n\n        `response.body` is set to opaque data structure `Req.Response.Async` which implements\n        `Enumerable` that receives and automatically parses messages. See module documentation\n        for example usage.\n\n        If the request is sent using HTTP/1, an extra process is spawned to consume messages\n        from the underlying socket. On both HTTP/1 and HTTP/2 the messages are sent to the\n        current process as soon as they arrive, as a firehose. If you wish to maximize request\n        rate or have more control over how messages are streamed, use `into: fun` or\n        `into: collectable` instead.\n\nResponse redirect options ([`redirect`](`Req.Steps.redirect/1`) step):\n\n  * `:redirect` - if set to `false`, disables automatic response redirects. Defaults to `true`.\n\n  * `:redirect_trusted` - by default, authorization credentials are only sent on redirects\n    with the same host, scheme and port. If `:redirect_trusted` is set to `true`, credentials\n    will be sent to any host.\n\n  * `:max_redirects` - the maximum number of redirects, defaults to `10`.\n\nRetry options ([`retry`](`Req.Steps.retry/1`) step):\n\n  * `:retry` - can be one of the following:\n\n      * `:safe_transient` (default) - retry safe (GET/HEAD) requests on one of:\n\n          * HTTP 408/429/500/502/503/504 responses\n\n          * `Req.TransportError` with `reason: :timeout | :econnrefused | :closed`\n\n          * `Req.HTTPError` with `protocol: :http2, reason: :unprocessed`\n\n      * `:transient` - same as `:safe_transient` except retries all HTTP methods (POST, DELETE, etc.)\n\n      * `fun` - a 2-arity function that accepts a `Req.Request` and either a `Req.Response` or an exception struct\n        and returns one of the following:\n\n          * `true` - retry with the default delay controller by default delay option described below.\n\n          * `{:delay, milliseconds}` - retry with the given delay.\n\n          * `false/nil` - don't retry.\n\n      * `false` - don't retry.\n\n  * `:retry_delay` - if not set, which is the default, the retry delay is determined by\n    the value of the `Retry-After` header on HTTP 429/503 responses. If the header is not set,\n    the default delay follows a simple exponential backoff: 1s, 2s, 4s, 8s, ...\n\n    `:retry_delay` can be set to a function that receives the retry count (starting at 0)\n    and returns the delay, the number of milliseconds to sleep before making another attempt.\n\n  * `:retry_log_level` - the log level to emit retry logs at. Can also be set to `false` to disable\n    logging these messages. Defaults to `:warning`.\n\n  * `:max_retries` - maximum number of retry attempts, defaults to `3` (for a total of `4`\n    requests to the server, including the initial one.)\n\nCaching options ([`cache`](`Req.Steps.cache/1`) step):\n\n  * `:cache` - if `true`, performs HTTP caching. Defaults to `false`.\n\n  * `:cache_dir` - the directory to store the cache, defaults to ` /req`\n    (see: `:filename.basedir/3`)\n\nRequest adapters:\n\n  * `:adapter` - adapter to use to make the actual HTTP request. See `:adapter` field description\n    in the `Req.Request` module documentation for more information.\n\n    The default is [`run_finch`](`Req.Steps.run_finch/1`).\n\n  * `:plug` - if set, calls the given plug instead of making an HTTP request over the network (via [`run_plug`](`Req.Steps.run_plug/1`) step).\n\n    The plug can be one of:\n\n      * A _function_ plug: a `fun(conn)` or `fun(conn, options)` function that takes a\n        `Plug.Conn` and returns a `Plug.Conn`.\n\n      * A _module_ plug: a `module` name or a `{module, options}` tuple.\n\nFinch options ([`run_finch`](`Req.Steps.run_finch/1`) step)\n\n  * `:finch` - the Finch pool to use. Defaults to pool automatically started by `Req`.\n\n  * `:connect_options` - dynamically starts (or re-uses already started) Finch pool with\n    the given connection options:\n\n      * `:timeout` - socket connect timeout in milliseconds, defaults to `30_000`.\n\n      * `:protocols` - the HTTP protocols to use, defaults to\n        `[:http1]`.\n\n      * `:hostname` - Mint explicit hostname.\n\n      * `:transport_opts` - Mint transport options.\n\n      * `:proxy_headers` - Mint proxy headers.\n\n      * `:proxy` - Mint HTTP/1 proxy settings, a `{schema, address, port, options}` tuple.\n\n      * `:client_settings` - Mint HTTP/2 client settings.\n\n  * `:inet6` - if set to true, uses IPv6. Defaults to `false`.\n\n  * `:pool_timeout` - pool checkout timeout in milliseconds, defaults to `5000`.\n\n  * `:receive_timeout` - socket receive timeout in milliseconds, defaults to `15_000`.\n\n  * `:unix_socket` - if set, connect through the given UNIX domain socket.\n\n  * `:pool_max_idle_time` - the maximum number of milliseconds that a pool can be\n    idle before being terminated, used only by HTTP1 pools. Default to `:infinity`.\n\n  * `:finch_private` - a map or keyword list of private metadata to add to the Finch request. May be useful\n    for adding custom data when handling telemetry with `Finch.Telemetry`.\n\n  * `:finch_request` - a function that executes the Finch request, defaults to using `Finch.request/3`.","ref":"Req.html#new/1-options"},{"type":"function","title":"Examples - Req.new/1","doc":"iex> req = Req.new(url: \"https://elixir-lang.org\")\n    iex> req.method\n    :get\n    iex> URI.to_string(req.url)\n    \"https://elixir-lang.org\"\n\nFake adapter:\n\n    iex> fake = fn request ->\n    ...>   {request, Req.Response.new(status: 200, body: \"it works!\")}\n    ...> end\n    iex>\n    iex> req = Req.new(adapter: fake)\n    iex> Req.get!(req).body\n    \"it works!\"","ref":"Req.html#new/1-examples"},{"type":"function","title":"Req.parse_message/2","doc":"Parses asynchronous response body message.\n\nA request with option `:into` set to `:self` returns response with asynchronous body.\nIn that case, Req sends chunks to the calling process as messages. You'd typically\nget them using `receive/1` or [`handle_info/2`](`c:GenServer.handle_info/2`) in a GenServer.\nThese messages should be parsed using this function. The possible return values are:\n\n  * `{:ok, chunks}` - where a chunk can be `{:data, binary}`, `{:trailers, trailers}`, or\n    `:done`.\n\n  * `{:error, reason}` - an error occured\n\n  * `:unknown` - the message was not meant for this response.\n\nSee also `Req.Response.Async`.","ref":"Req.html#parse_message/2"},{"type":"function","title":"Examples - Req.parse_message/2","doc":"iex> resp = Req.get!(\"http://httpbin.org/stream/2\", into: :self)\n    iex> Req.parse_message(resp, receive do message -> message end)\n    {:ok, [data: \"{\"url\": \"http://httpbin.org/stream/2\", ..., \"id\": 0}\\n\"]}\n    iex> Req.parse_message(resp, receive do message -> message end)\n    {:ok, [data: \"{\"url\": \"http://httpbin.org/stream/2\", ..., \"id\": 1}\\n\"]}\n    iex> Req.parse_message(resp, receive do message -> message end)\n    {:ok, [:done]}\n    iex> Req.parse_message(resp, :other)\n    :unknown","ref":"Req.html#parse_message/2-examples"},{"type":"function","title":"Req.patch/2","doc":"Makes a PATCH request and returns a response or an error.\n\n`request` can be one of:\n\n  * an url (`String` or `URI`);\n\n  * a `Keyword` options;\n\n  * a `Req.Request` struct\n\nSee `new/1` for a list of available options.","ref":"Req.html#patch/2"},{"type":"function","title":"Examples - Req.patch/2","doc":"With URL:\n\n    iex> {:ok, resp} = Req.patch(\"https://httpbin.org/anything\", body: \"hello!\")\n    iex> resp.body[\"data\"]\n    \"hello!\"\n\nWith options:\n\n    iex> {:ok, resp} = Req.patch(url: \"https://httpbin.org/anything\", body: \"hello!\")\n    iex> resp.body[\"data\"]\n    \"hello!\"\n\nWith request struct:\n\n    iex> req = Req.new(url: \"https://httpbin.org/anything\")\n    iex> {:ok, resp} = Req.patch(req, body: \"hello!\")\n    iex> resp.body[\"data\"]\n    \"hello!\"","ref":"Req.html#patch/2-examples"},{"type":"function","title":"Req.patch!/2","doc":"Makes a PATCH request and returns a response or raises an error.\n\n`request` can be one of:\n\n  * an url (`String` or `URI`);\n\n  * a `Keyword` options;\n\n  * a `Req.Request` struct\n\nSee `new/1` for a list of available options.","ref":"Req.html#patch!/2"},{"type":"function","title":"Examples - Req.patch!/2","doc":"With URL:\n\n    iex> Req.patch!(\"https://httpbin.org/anything\", body: \"hello!\").body[\"data\"]\n    \"hello!\"\n\nWith options:\n\n    iex> Req.patch!(url: \"https://httpbin.org/anything\", body: \"hello!\").body[\"data\"]\n    \"hello!\"\n\nWith request struct:\n\n    iex> req = Req.new(url: \"https://httpbin.org/anything\")\n    iex> Req.patch!(req, body: \"hello!\").body[\"data\"]\n    \"hello!\"","ref":"Req.html#patch!/2-examples"},{"type":"function","title":"Req.post/2","doc":"Makes a POST request and returns a response or an error.\n\n`request` can be one of:\n\n  * an url (`String` or `URI`);\n\n  * a `Keyword` options;\n\n  * a `Req.Request` struct\n\nSee `new/1` for a list of available options.","ref":"Req.html#post/2"},{"type":"function","title":"Examples - Req.post/2","doc":"With URL:\n\n    iex> {:ok, resp} = Req.post(\"https://httpbin.org/anything\", body: \"hello!\")\n    iex> resp.body[\"data\"]\n    \"hello!\"\n\n    iex> {:ok, resp} = Req.post(\"https://httpbin.org/anything\", form: [x: 1])\n    iex> resp.body[\"form\"]\n    %{\"x\" => \"1\"}\n\n    iex> {:ok, resp} = Req.post(\"https://httpbin.org/anything\", json: %{x: 2})\n    iex> resp.body[\"json\"]\n    %{\"x\" => 2}\n\nWith options:\n\n    iex> {:ok, resp} = Req.post(url: \"https://httpbin.org/anything\", body: \"hello!\")\n    iex> resp.body[\"data\"]\n    \"hello!\"\n\nWith request struct:\n\n    iex> req = Req.new(url: \"https://httpbin.org/anything\")\n    iex> {:ok, resp} = Req.post(req, body: \"hello!\")\n    iex> resp.body[\"data\"]\n    \"hello!\"","ref":"Req.html#post/2-examples"},{"type":"function","title":"Req.post!/2","doc":"Makes a POST request and returns a response or raises an error.\n\n`request` can be one of:\n\n  * an url (`String` or `URI`);\n\n  * a `Keyword` options;\n\n  * a `Req.Request` struct\n\nSee `new/1` for a list of available options.","ref":"Req.html#post!/2"},{"type":"function","title":"Examples - Req.post!/2","doc":"With URL:\n\n    iex> Req.post!(\"https://httpbin.org/anything\", body: \"hello!\").body[\"data\"]\n    \"hello!\"\n\n    iex> Req.post!(\"https://httpbin.org/anything\", form: [x: 1]).body[\"form\"]\n    %{\"x\" => \"1\"}\n\n    iex> Req.post!(\"https://httpbin.org/anything\", json: %{x: 2}).body[\"json\"]\n    %{\"x\" => 2}\n\nWith options:\n\n    iex> Req.post!(url: \"https://httpbin.org/anything\", body: \"hello!\").body[\"data\"]\n    \"hello!\"\n\nWith request struct:\n\n    iex> req = Req.new(url: \"https://httpbin.org/anything\")\n    iex> Req.post!(req, body: \"hello!\").body[\"data\"]\n    \"hello!\"","ref":"Req.html#post!/2-examples"},{"type":"function","title":"Req.put/2","doc":"Makes a PUT request and returns a response or an error.\n\n`request` can be one of:\n\n  * an url (`String` or `URI`);\n\n  * a `Keyword` options;\n\n  * a `Req.Request` struct\n\nSee `new/1` for a list of available options.","ref":"Req.html#put/2"},{"type":"function","title":"Examples - Req.put/2","doc":"With URL:\n\n    iex> {:ok, resp} = Req.put(\"https://httpbin.org/anything\", body: \"hello!\")\n    iex> resp.body[\"data\"]\n    \"hello!\"\n\nWith options:\n\n    iex> {:ok, resp} = Req.put(url: \"https://httpbin.org/anything\", body: \"hello!\")\n    iex> resp.body[\"data\"]\n    \"hello!\"\n\nWith request struct:\n\n    iex> req = Req.new(url: \"https://httpbin.org/anything\")\n    iex> {:ok, resp} = Req.put(req, body: \"hello!\")\n    iex> resp.body[\"data\"]\n    \"hello!\"","ref":"Req.html#put/2-examples"},{"type":"function","title":"Req.put!/2","doc":"Makes a PUT request and returns a response or raises an error.\n\n`request` can be one of:\n\n  * an url (`String` or `URI`);\n\n  * a `Keyword` options;\n\n  * a `Req.Request` struct\n\nSee `new/1` for a list of available options.","ref":"Req.html#put!/2"},{"type":"function","title":"Examples - Req.put!/2","doc":"With URL:\n\n    iex> Req.put!(\"https://httpbin.org/anything\", body: \"hello!\").body[\"data\"]\n    \"hello!\"\n\nWith options:\n\n    iex> Req.put!(url: \"https://httpbin.org/anything\", body: \"hello!\").body[\"data\"]\n    \"hello!\"\n\nWith request struct:\n\n    iex> req = Req.new(url: \"https://httpbin.org/anything\")\n    iex> Req.put!(req, body: \"hello!\").body[\"data\"]\n    \"hello!\"","ref":"Req.html#put!/2-examples"},{"type":"function","title":"Req.request/2","doc":"Makes an HTTP request and returns a response or an error.\n\n`request` can be one of:\n\n  * a `Keyword` options;\n  * a `Req.Request` struct\n\nSee `new/1` for a list of available options.\n\nAlso see `run/2` for a similar function that returns the request and the response or error.","ref":"Req.html#request/2"},{"type":"function","title":"Examples - Req.request/2","doc":"With options keywords list:\n\n    iex> {:ok, response} = Req.request(url: \"https://api.github.com/repos/wojtekmach/req\")\n    iex> response.status\n    200\n    iex> response.body[\"description\"]\n    \"Req is a batteries-included HTTP client for Elixir.\"\n\nWith request struct:\n\n    iex> req = Req.new(url: \"https://api.github.com/repos/elixir-lang/elixir\")\n    iex> {:ok, response} = Req.request(req)\n    iex> response.status\n    200","ref":"Req.html#request/2-examples"},{"type":"function","title":"Req.request!/2","doc":"Makes an HTTP request and returns a response or raises an error.\n\nSee `new/1` for a list of available options.\n\nAlso see `run!/2` for a similar function that returns the request and the response or error.","ref":"Req.html#request!/2"},{"type":"function","title":"Examples - Req.request!/2","doc":"With options keywords list:\n\n    iex> Req.request!(url: \"https://api.github.com/repos/elixir-lang/elixir\").status\n    200\n\nWith request struct:\n\n    iex> req = Req.new(url: \"https://api.github.com/repos/elixir-lang/elixir\")\n    iex> Req.request!(req).status\n    200","ref":"Req.html#request!/2-examples"},{"type":"function","title":"Req.run/2","doc":"Makes an HTTP request and returns the request and response or error.\n\n`request` can be one of:\n\n  * an url (`String` or `URI`);\n\n  * a `Keyword` options;\n\n  * a `Req.Request` struct\n\nSee `new/1` for a list of available options.\n\nAlso see `request/2` for a similar function that returns the response or error\n(without the request).","ref":"Req.html#run/2"},{"type":"function","title":"Examples - Req.run/2","doc":"With options keywords list:\n\n    iex> {req, resp} = Req.run(url: \"https://api.github.com/repos/elixir-lang/elixir\")\n    iex> req.url.host\n    \"api.github.com\"\n    iex> resp.status\n    200\n\nWith request struct and options:\n\n    iex> req = Req.new(base_url: \"https://api.github.com\")\n    iex> {req, resp} = Req.run(req, url: \"/repos/elixir-lang/elixir\")\n    iex> req.url.host\n    \"api.github.com\"\n    iex> resp.status\n    200\n\nReturns an error:\n\n    iex> {_req, exception} = Req.run(\"http://localhost:9999\", retry: false)\n    iex> exception\n    %Req.TransportError{reason: :econnrefused}","ref":"Req.html#run/2-examples"},{"type":"function","title":"Req.run!/2","doc":"Makes an HTTP request and returns the request and response or raises on errors.\n\n`request` can be one of:\n\n  * an url (`String` or `URI`);\n\n  * a `Keyword` options;\n\n  * a `Req.Request` struct\n\nSee `new/1` for a list of available options.\n\nAlso see `request!/2` for a similar function that returns the response (without the request).","ref":"Req.html#run!/2"},{"type":"function","title":"Examples - Req.run!/2","doc":"With options keywords list:\n\n    iex> {req, resp} = Req.run!(url: \"https://api.github.com/repos/elixir-lang/elixir\")\n    iex> req.url.host\n    \"api.github.com\"\n    iex> resp.status\n    200\n\nWith request struct and options:\n\n    iex> req = Req.new(base_url: \"https://api.github.com\")\n    iex> {req, resp} = Req.run!(req, url: \"/repos/elixir-lang/elixir\")\n    iex> req.url.host\n    \"api.github.com\"\n    iex> resp.status\n    200\n\nRaises an error:\n\n    iex> Req.run!(\"http://localhost:9999\", retry: false)\n    ** (Req.TransportError) connection refused","ref":"Req.html#run!/2-examples"},{"type":"type","title":"Req.url/0","doc":"","ref":"Req.html#t:url/0"},{"type":"module","title":"Req.Request","doc":"The low-level API and the request struct.\n\nReq is composed of:\n\n  * `Req` - the high-level API\n\n  * `Req.Request` - the low-level API and the request struct (you're here!)\n\n  * `Req.Steps` - the collection of built-in steps\n\n  * `Req.Test` - the testing conveniences\n\nThe low-level API and the request struct is the foundation of Req's extensibility. Virtually all\nof the functionality is broken down into individual pieces - steps. Req works by running the\nrequest struct through these steps. You can easily reuse or rearrange built-in steps or write new\nones.\n\nTo make using custom steps by others even easier, they can be packaged up into plugins.\nSee [\"Writing Plugins\"](#module-writing-plugins) section for more information.","ref":"Req.Request.html"},{"type":"module","title":"The Low-level API - Req.Request","doc":"Most Req users would use it like this:\n\n    Req.get!(\"https://api.github.com/repos/wojtekmach/req\").body[\"description\"]\n    #=> \"Req is a batteries-included HTTP client for Elixir.\"\n\nHere is the equivalent using the low-level API:\n\n    url = \"https://api.github.com/repos/wojtekmach/req\"\n\n    req =\n      Req.Request.new(method: :get, url: url)\n      |> Req.Request.append_request_steps(\n        put_user_agent: &Req.Steps.put_user_agent/1,\n        # ...\n      )\n      |> Req.Request.append_response_steps(\n        # ...\n        decompress_body: &Req.Steps.decompress_body/1,\n        decode_body: &Req.Steps.decode_body/1,\n        # ...\n      )\n      |> Req.Request.append_error_steps(\n        retry: &Req.Steps.retry/1,\n        # ...\n      )\n\n    {req, resp} = Req.Request.run_request(req)\n    resp.body[\"description\"]\n    #=> \"Req is a batteries-included HTTP client for Elixir.\"\n\nBy putting the request pipeline yourself you have precise control of exactly what is running and in what order.","ref":"Req.Request.html#module-the-low-level-api"},{"type":"module","title":"The Request Struct - Req.Request","doc":"Public fields are:\n\n  * `:method` - the HTTP request method.\n\n  * `:url` - the HTTP request URL.\n\n  * `:headers` - the HTTP request headers. The header names should be downcased.\n    See also \"Header Names\" section in `Req` module documentation.\n\n  * `:body` - the HTTP request body.\n\n    Can be one of:\n\n      * `iodata` - eagerly send request body\n\n      * `enumerable` - stream request body\n\n  * `:into` - where to send the response body. It can be one of:\n\n      * `nil` - (default) read the whole response body and store it in the `response.body`\n        field.\n\n      * `fun` - stream response body using a function. The first argument is a `{:data, data}`\n        tuple containing the chunk of the response body. The second argument is a\n        `{request, response}` tuple. To continue streaming chunks, return `{:cont, {req, resp}}`.\n        To cancel, return `{:halt, {req, resp}}`. For example:\n\n            into: fn {:data, data}, {req, resp} ->\n              IO.puts(data)\n              {:cont, {req, resp}}\n            end\n\n      * `collectable` - stream response body into a `t:Collectable.t/0`. For example:\n\n            into: File.stream!(\"path\")\n\n        Note that the collectable is only used, if the response status is 200. In other cases,\n        the body is accumulated and processed as usual.\n\n  * `:options` - the options to be used by steps. The exact representation of options is private.\n    Calling `request.options[key]`, `put_in(request.options[key], value)`, and\n    `update_in(request.options[key], fun)` is allowed. `get_option/3` and `delete_option/2`\n    are also available for additional ways to manipulate the internal representation.\n\n  * `:halted` - whether the request pipeline is halted. See `halt/2`.\n\n  * `:adapter` - a request step that makes the actual HTTP request. Defaults to\n    `Req.Steps.run_finch/1`. See [\"Adapter\"](#module-adapter) section below for more information.\n\n  * `:request_steps` - the list of request steps\n\n  * `:response_steps` - the list of response steps\n\n  * `:error_steps` - the list of error steps\n\n  * `:private` - a map reserved for libraries and frameworks to use.\n    The keys must be atoms. Prefix the keys with the name of your project\n    to avoid any future conflicts. The `req_` prefix is reserved for Req.","ref":"Req.Request.html#module-the-request-struct"},{"type":"module","title":"Steps - Req.Request","doc":"Req has three types of steps: request, response, and error.\n\nRequest steps are used to refine the data that will be sent to the server.\n\nAfter making the actual HTTP request, we'll either get a HTTP response or an error.\nThe request, along with the response or error, will go through response or\nerror steps, respectively.\n\nNothing is actually executed until we run the pipeline with `Req.Request.run_request/1`.","ref":"Req.Request.html#module-steps"},{"type":"module","title":"Request Steps - Req.Request","doc":"A **request step** (`t:request_step/0`) is a function that accepts a `request` and returns one\nof the following:\n\n  * A `request`.\n\n  * A `{request, response_or_error}` tuple. In this case no further request steps are executed\n    and the return value goes through response or error steps.\n\n#### Examples\n\nA request step that adds a `user-agent` header if it's not there already:\n\n    def put_default_headers(request) do\n      Req.Request.put_new_header(request, \"user-agent\", \"req\")\n    end\n\nThe next is a request step that reads the response from cache if available. Note how, if the\ncached response is available, this step returns a `{request, response}` tuple so that the\nrequest doesn't actually go through:\n\n    def read_from_cache(request) do\n      case ResponseCache.fetch(request) do\n        {:ok, response} -> {request, response}\n        :error -> request\n      end\n    end","ref":"Req.Request.html#module-request-steps"},{"type":"module","title":"Response and Error Steps - Req.Request","doc":"A response step (`t:response_step/0`) is a function that accepts a `{request, response}` tuple\nand returns one of the following:\n\n  * A `{request, response}` tuple.\n\n  * A `{request, exception}` tuple. In that case, no further response steps are executed but the\n    exception goes through error steps.\n\nSimilarly, an error step is a function that accepts a `{request, exception}` tuple and returns one\nof the following:\n\n  * A `{request, exception}` tuple\n\n  * A `{request, response}` tuple. In that case, no further error steps are executed but the\n    response goes through response steps.\n\nExamples:\n\n    def decode({request, response}) do\n      case Req.Response.get_header(response, \"content-type\") do\n        [\"application/json\" <> _] ->\n          {request, update_in(response.body, &Jason.decode!/1)}\n\n        [] ->\n          {request, response}\n      end\n    end\n\n    def log_error({request, exception}) do\n      Logger.error([\"#{request.method} #{request.uri}: \", Exception.message(exception)])\n      {request, exception}\n    end","ref":"Req.Request.html#module-response-and-error-steps"},{"type":"module","title":"Halting - Req.Request","doc":"Any step can call `halt/2` to halt the pipeline. This prevents any further steps\nfrom being invoked.\n\nExamples:\n\n    def circuit_breaker(request) do\n      if CircuitBreaker.open?() do\n        Req.Request.halt(request, RuntimeError.exception(\"circuit breaker is open\"))\n      else\n        request\n      end\n    end","ref":"Req.Request.html#module-halting"},{"type":"module","title":"Writing Plugins - Req.Request","doc":"Custom steps can be packaged into plugins so that they are even easier to use by others.\n\nHere's an example plugin:\n\n    defmodule PrintHeaders do\n      @doc \"\"\"\n      Prints request and response headers.","ref":"Req.Request.html#module-writing-plugins"},{"type":"module","title":"Request Options - Req.Request","doc":"* `:print_headers` - if `true`, prints the headers. Defaults to `false`.\n\n      \"\"\"\n      def attach(%Req.Request{} = request, options \\\\ []) do\n        request\n        |> Req.Request.register_options([:print_headers])\n        |> Req.Request.merge_options(options)\n        |> Req.Request.append_request_steps(print_headers: &print_request_headers/1)\n        |> Req.Request.prepend_response_steps(print_headers: &print_response_headers/1)\n      end\n\n      defp print_request_headers(request) do\n        if request.options[:print_headers] do\n          print_headers(\"> \", request.headers)\n        end\n\n        request\n      end\n\n      defp print_response_headers({request, response}) do\n        if request.options[:print_headers] do\n          print_headers(\"< \", response.headers)\n        end\n\n        {request, response}\n      end\n\n      defp print_headers(prefix, headers) do\n        for {name, value} <- headers do\n          IO.puts([prefix, name, \": \", value])\n        end\n      end\n    end\n\nAnd here is how we can use it:\n\n    req = Req.new() |> PrintHeaders.attach()\n\n    Req.get!(req, url: \"https://httpbin.org/json\").status\n    200\n\n    Req.get!(req, url: \"https://httpbin.org/json\", print_headers: true).status\n    # Outputs:\n    # > accept-encoding: br, gzip\n    # > user-agent: req/0.3.0-dev\n    #   PrintHeaders.attach(print_headers: true)\n    Req.get!(req, url: \"https://httpbin.org/json\").status\n    # Outputs:\n    # > accept-encoding: br, gzip\n    # ...\n    200\n\nAs you can see a plugin is simply a module. While this is not enforced, the plugin should follow\nthese conventions:\n\n  * It should export an `attach/1` function that takes and returns the request struct\n\n  * The attach functions mostly just adds steps and it is the steps that do the actual work\n\n  * A user should be able to attach your plugin alongside other plugins. For this reason,\n    plugin functionality should usually only happen on a specific \"trigger\": on a specific\n    option, on a specific URL scheme or host, etc. This is especially important for plugins\n    that perform authentication; you don't want to accidentally expose a token from service A\n    when a user makes request to service B.\n\n  * If your plugin supports custom options, register them with `register_options/2`\n\n  * Sometimes it is useful to pass options when attaching the plugin. For that, export an\n    `attach/2` function and call `merge_options/2`. Remember to first register\n    options before merging!","ref":"Req.Request.html#module-request-options"},{"type":"module","title":"Adapter - Req.Request","doc":"As noted in the [\"Request Steps\"](#module-request-steps) section, a request step besides returning the request,\nmight also return `{request, response}` or `{request, exception}`, thus invoking either response or error steps next.\nThis is exactly how Req makes the underlying HTTP call, by invoking a request step that follows this contract.\n\nThe default adapter is using Finch via the `Req.Steps.run_finch/1` step.\n\nHere is a mock adapter that always returns a successful response:\n\n    adapter = fn request ->\n      response = %Req.Response{status: 200, body: \"it works!\"}\n      {request, response}\n    end\n\n    Req.request!(url: \"http://example\", adapter: adapter).body\n    #=> \"it works!\"\n\nHere is another one that uses the `Req.Response.json/2` function to conveniently\nreturn a JSON response:\n\n    adapter = fn request ->\n      response = Req.Response.json(%{hello: 42})\n      {request, response}\n    end\n\n    resp = Req.request!(url: \"http://example\", adapter: adapter)\n    resp.headers\n    #=> [{\"content-type\", \"application/json\"}]\n    resp.body\n    #=> %{\"hello\" => 42}\n\nAnd here is a naive Hackney-based adapter:\n\n    hackney = fn request ->\n      case :hackney.request(\n             request.method,\n             URI.to_string(request.url),\n             request.headers,\n             request.body,\n             [:with_body]\n           ) do\n        {:ok, status, headers, body} ->\n          headers = for {name, value} <- headers, do: {String.downcase(name, :ascii), value}\n          response = %Req.Response{status: status, headers: headers, body: body}\n          {request, response}\n\n        {:error, reason} ->\n          {request, RuntimeError.exception(inspect(reason))}\n      end\n    end\n\n    Req.get!(\"https://api.github.com/repos/wojtekmach/req\", adapter: hackney).body[\"description\"]\n    #=> \"Req is a batteries-included HTTP client for Elixir.\"","ref":"Req.Request.html#module-adapter"},{"type":"function","title":"Req.Request.append_error_steps/2","doc":"Appends **error steps** to the existing error steps.\n\nSee the [\"Response and Error Steps\"](#module-response-and-error-steps) section in the\nmodule documentation for more information.","ref":"Req.Request.html#append_error_steps/2"},{"type":"function","title":"Examples - Req.Request.append_error_steps/2","doc":"Req.Request.append_error_steps(request,\n      noop: fn {request, exception} -> {request, exception} end,\n      inspect: &IO.inspect/1\n    )","ref":"Req.Request.html#append_error_steps/2-examples"},{"type":"function","title":"Req.Request.append_request_steps/2","doc":"Appends **request steps** to the existing request steps.\n\nSee the [\"Request Steps\"](#module-request-steps) section in the module documentation\nfor more information.","ref":"Req.Request.html#append_request_steps/2"},{"type":"function","title":"Examples - Req.Request.append_request_steps/2","doc":"Req.Request.append_request_steps(request,\n      noop: fn request -> request end,\n      inspect: &IO.inspect/1\n    )","ref":"Req.Request.html#append_request_steps/2-examples"},{"type":"function","title":"Req.Request.append_response_steps/2","doc":"Appends **response steps** to the existing response steps.\n\nSee the [\"Response and Error Steps\"](#module-response-and-error-steps) section in the\nmodule documentation for more information.","ref":"Req.Request.html#append_response_steps/2"},{"type":"function","title":"Examples - Req.Request.append_response_steps/2","doc":"Req.Request.append_response_steps(request,\n      noop: fn {request, response} -> {request, response} end,\n      inspect: &IO.inspect/1\n    )","ref":"Req.Request.html#append_response_steps/2-examples"},{"type":"function","title":"Req.Request.delete_header/2","doc":"Deletes the header given by `name`.\n\nAll occurrences of the header are deleted, in case the header is repeated multiple times.\n\nSee also \"Header Names\" section in `Req` module documentation.","ref":"Req.Request.html#delete_header/2"},{"type":"function","title":"Examples - Req.Request.delete_header/2","doc":"iex> Req.Request.get_header(req, \"cache-control\")\n    [\"max-age=600\", \"no-transform\"]\n    iex> req = Req.Request.delete_header(req, \"cache-control\")\n    iex> Req.Request.get_header(req, \"cache-control\")\n    []","ref":"Req.Request.html#delete_header/2-examples"},{"type":"function","title":"Req.Request.delete_option/2","doc":"Deletes the given option `key`.","ref":"Req.Request.html#delete_option/2"},{"type":"function","title":"Examples - Req.Request.delete_option/2","doc":"iex> req = Req.Request.new(options: [a: 1])\n    iex> Req.Request.get_option(req, :a)\n    1\n    iex> req = Req.Request.delete_option(req, :a)\n    iex> Req.Request.get_option(req, :a)\n    nil","ref":"Req.Request.html#delete_option/2-examples"},{"type":"function","title":"Req.Request.drop_options/2","doc":"Drops the given `keys` from options.","ref":"Req.Request.html#drop_options/2"},{"type":"function","title":"Examples - Req.Request.drop_options/2","doc":"iex> req = Req.Request.new(options: [a: 1, b: 2, c: 3])\n    iex> req = Req.Request.drop_options(req, [:a, :b])\n    iex> Req.Request.get_option(req, :a)\n    nil\n    iex> Req.Request.get_option(req, :c)\n    3","ref":"Req.Request.html#drop_options/2-examples"},{"type":"function","title":"Req.Request.fetch_option/2","doc":"Fetches the value for the option `key`.\n\nSee also `get_option/3`.","ref":"Req.Request.html#fetch_option/2"},{"type":"function","title":"Examples - Req.Request.fetch_option/2","doc":"iex> req = Req.Request.new(options: [a: 1])\n    iex> Req.Request.fetch_option(req, :a)\n    {:ok, 1}\n    iex> Req.Request.fetch_option(req, :b)\n    :error","ref":"Req.Request.html#fetch_option/2-examples"},{"type":"function","title":"Req.Request.fetch_option!/2","doc":"Fetches the value for the option `key` or raises if it's not set.\n\nSee also `get_option/3`.","ref":"Req.Request.html#fetch_option!/2"},{"type":"function","title":"Examples - Req.Request.fetch_option!/2","doc":"iex> req = Req.Request.new(options: [a: 1])\n    iex> Req.Request.fetch_option!(req, :a)\n    1\n    iex> Req.Request.fetch_option!(req, :b)\n    ** (KeyError) option :b is not set","ref":"Req.Request.html#fetch_option!/2-examples"},{"type":"function","title":"Req.Request.get_header/2","doc":"Returns the values of the header specified by `name`.\n\nSee also \"Header Names\" section in `Req` module documentation.","ref":"Req.Request.html#get_header/2"},{"type":"function","title":"Examples - Req.Request.get_header/2","doc":"iex> req = Req.new(headers: [{\"accept\", \"application/json\"}])\n    iex> Req.Request.get_header(req, \"accept\")\n    [\"application/json\"]\n    iex> Req.Request.get_header(req, \"x-unknown\")\n    []","ref":"Req.Request.html#get_header/2-examples"},{"type":"function","title":"Req.Request.get_option/3","doc":"Gets the value for the option `key`.\n\nSee also `fetch_option!/2`.","ref":"Req.Request.html#get_option/3"},{"type":"function","title":"Examples - Req.Request.get_option/3","doc":"iex> req = Req.Request.new(options: [a: 1])\n    iex> Req.Request.get_option(req, :a)\n    1\n    iex> Req.Request.get_option(req, :b)\n    nil\n    iex> Req.Request.get_option(req, :b, 0)\n    0","ref":"Req.Request.html#get_option/3-examples"},{"type":"function","title":"Req.Request.get_option_lazy/3","doc":"Gets the value for the option `key`.\n\nThis is useful if the default value is very expensive to calculate or generally\ndifficult to setup and teardown again.\n\nSee also `get_option/3`.","ref":"Req.Request.html#get_option_lazy/3"},{"type":"function","title":"Examples - Req.Request.get_option_lazy/3","doc":"iex> req = Req.Request.new(options: [a: 1])\n    iex> fun = fn ->\n    ...>   # some expensive operation here\n    ...>   42\n    ...> end\n    iex> Req.Request.get_option_lazy(req, :a, fun)\n    1\n    iex> Req.Request.get_option_lazy(req, :b, fun)\n    42","ref":"Req.Request.html#get_option_lazy/3-examples"},{"type":"function","title":"Req.Request.get_private/3","doc":"Gets the value for a specific private `key`.","ref":"Req.Request.html#get_private/3"},{"type":"function","title":"Req.Request.halt/2","doc":"Halts the request pipeline preventing any further steps from executing.\n\nThis function returns an updated request and the response or exception that caused the halt.\nIt's perfect when used in a request step to stop the pipeline.\n\nSee the [\"Halting\"](#module-halting) section in the module documentation for more information.","ref":"Req.Request.html#halt/2"},{"type":"function","title":"Examples - Req.Request.halt/2","doc":"Req.Request.prepend_request_steps(request, circuit_breaker: fn request ->\n      if CircuitBreaker.open?() do\n        Req.Request.halt(request, RuntimeError.exception(\"circuit breaker is open\"))\n      else\n        request\n      end\n    end)","ref":"Req.Request.html#halt/2-examples"},{"type":"function","title":"Req.Request.merge_options/2","doc":"Merges given options into the request.","ref":"Req.Request.html#merge_options/2"},{"type":"function","title":"Examples - Req.Request.merge_options/2","doc":"iex> req = Req.new(auth: {:basic, \"alice:secret\"}, http_errors: :raise)\n    iex> req = Req.Request.merge_options(req, auth: {:bearer, \"abcd\"}, base_url: \"https://example.com\")\n    iex> req.options[:auth]\n    {:bearer, \"abcd\"}\n    iex> req.options[:http_errors]\n    :raise\n    iex> req.options[:base_url]\n    \"https://example.com\"","ref":"Req.Request.html#merge_options/2-examples"},{"type":"function","title":"Req.Request.new/1","doc":"Returns a new request struct.","ref":"Req.Request.html#new/1"},{"type":"function","title":"Options - Req.Request.new/1","doc":"* `:method` - the request method, defaults to `:get`.\n\n  * `:url` - the request URL.\n\n  * `:headers` - the request headers, defaults to `[]`.\n\n  * `:body` - the request body, defaults to `nil`.\n\n  * `:adapter` - the request adapter, defaults to calling [`run_finch`](`Req.Steps.run_finch/1`).","ref":"Req.Request.html#new/1-options"},{"type":"function","title":"Examples - Req.Request.new/1","doc":"iex> req = Req.Request.new(url: \"https://api.github.com/repos/wojtekmach/req\")\n    iex> {req, resp} = Req.Request.run_request(req)\n    iex> req.url.host\n    \"api.github.com\"\n    iex> resp.status\n    200","ref":"Req.Request.html#new/1-examples"},{"type":"function","title":"Req.Request.prepend_error_steps/2","doc":"Prepends **error steps** to the existing error steps.\n\nSee the [\"Response and Error Steps\"](#module-response-and-error-steps) section in the\nmodule documentation for more information.","ref":"Req.Request.html#prepend_error_steps/2"},{"type":"function","title":"Examples - Req.Request.prepend_error_steps/2","doc":"Req.Request.prepend_error_steps(request,\n      noop: fn {request, exception} -> {request, exception} end,\n      inspect: &IO.inspect/1\n    )","ref":"Req.Request.html#prepend_error_steps/2-examples"},{"type":"function","title":"Req.Request.prepend_request_steps/2","doc":"Prepends **request steps** to the existing request steps.\n\nSee the [\"Request Steps\"](#module-request-steps) section in the module documentation\nfor more information.","ref":"Req.Request.html#prepend_request_steps/2"},{"type":"function","title":"Examples - Req.Request.prepend_request_steps/2","doc":"Req.Request.prepend_request_steps(request,\n      noop: fn request -> request end,\n      inspect: &IO.inspect/1\n    )","ref":"Req.Request.html#prepend_request_steps/2-examples"},{"type":"function","title":"Req.Request.prepend_response_steps/2","doc":"Prepends **response steps** to the existing response steps.\n\nSee the [\"Response and Error Steps\"](#module-response-and-error-steps) section in the\nmodule documentation for more information.","ref":"Req.Request.html#prepend_response_steps/2"},{"type":"function","title":"Examples - Req.Request.prepend_response_steps/2","doc":"Req.Request.prepend_response_steps(request,\n      noop: fn {request, response} -> {request, response} end,\n      inspect: &IO.inspect/1\n    )","ref":"Req.Request.html#prepend_response_steps/2-examples"},{"type":"function","title":"Req.Request.put_header/3","doc":"Sets the header `name` to `value`.\n\nThe value can be a binary or a list of binaries,\n\nIf the header was previously set, its value is overwritten.\n\nSee also \"Header Names\" section in `Req` module documentation.","ref":"Req.Request.html#put_header/3"},{"type":"function","title":"Examples - Req.Request.put_header/3","doc":"iex> req = Req.new()\n    iex> Req.Request.get_header(req, \"accept\")\n    []\n    iex> req = Req.Request.put_header(req, \"accept\", \"application/json\")\n    iex> Req.Request.get_header(req, \"accept\")\n    [\"application/json\"]","ref":"Req.Request.html#put_header/3-examples"},{"type":"function","title":"Req.Request.put_headers/2","doc":"Adds (or replaces) multiple request headers.\n\nSee `put_header/3` for more information.","ref":"Req.Request.html#put_headers/2"},{"type":"function","title":"Examples - Req.Request.put_headers/2","doc":"iex> req = Req.new()\n    iex> req = Req.Request.put_headers(req, [{\"accept\", \"text/html\"}, {\"accept-encoding\", \"gzip\"}])\n    iex> Req.Request.get_header(req, \"accept\")\n    [\"text/html\"]\n    iex> Req.Request.get_header(req, \"accept-encoding\")\n    [\"gzip\"]","ref":"Req.Request.html#put_headers/2-examples"},{"type":"function","title":"Req.Request.put_new_header/3","doc":"Adds a request header `name` unless already present.\n\nSee `put_header/3` for more information.","ref":"Req.Request.html#put_new_header/3"},{"type":"function","title":"Examples - Req.Request.put_new_header/3","doc":"iex> req =\n    ...>   Req.new()\n    ...>   |> Req.Request.put_new_header(\"accept\", \"application/json\")\n    ...>   |> Req.Request.put_new_header(\"accept\", \"application/html\")\n    iex> Req.Request.get_header(req, \"accept\")\n    [\"application/json\"]","ref":"Req.Request.html#put_new_header/3-examples"},{"type":"function","title":"Req.Request.put_private/3","doc":"Assigns a private `key` to `value`.","ref":"Req.Request.html#put_private/3"},{"type":"function","title":"Req.Request.register_options/2","doc":"Registers options to be used by a custom steps.\n\nReq ensures that all used options were previously registered which helps\nfinding accidentally mistyped option names. If you're adding custom steps\nthat are accepting options, call this function to register them.","ref":"Req.Request.html#register_options/2"},{"type":"function","title":"Examples - Req.Request.register_options/2","doc":"iex> Req.request!(urll: \"https://httpbin.org\")\n    ** (ArgumentError) unknown option :urll. Did you mean :url?\n\n    iex> Req.new(bas_url: \"https://httpbin.org\")\n    ** (ArgumentError) unknown option :bas_url. Did you mean :base_url?\n\n    req =\n      Req.new(base_url: \"https://httpbin.org\")\n      |> Req.Request.register_options([:foo])\n\n    Req.get!(req, url: \"/status/201\", foo: :bar).status\n    #=> 201","ref":"Req.Request.html#register_options/2-examples"},{"type":"function","title":"Req.Request.run/1","doc":"","ref":"Req.Request.html#run/1"},{"type":"function","title":"Req.Request.run!/1","doc":"","ref":"Req.Request.html#run!/1"},{"type":"function","title":"Req.Request.run_request/1","doc":"Runs the request pipeline.\n\nReturns `{request, response}` or `{request, exception}`.","ref":"Req.Request.html#run_request/1"},{"type":"function","title":"Examples - Req.Request.run_request/1","doc":"iex> req = Req.Request.new(url: \"https://api.github.com/repos/wojtekmach/req\")\n    iex> {request, response} = Req.Request.run_request(req)\n    iex> request.url.host\n    \"api.github.com\"\n    iex> response.status\n    200","ref":"Req.Request.html#run_request/1-examples"},{"type":"function","title":"Req.Request.update_private/4","doc":"Updates private `key` with the given function.\n\nIf `key` is present in request private map then the existing value is passed to `fun` and its\nresult is used as the updated value of `key`. If `key` is not present, `default` is inserted\nas the value of `key`. The default value will not be passed through the update function.","ref":"Req.Request.html#update_private/4"},{"type":"function","title":"Examples - Req.Request.update_private/4","doc":"iex> req = %Req.Request{private: %{a: 1}}\n    iex> Req.Request.update_private(req, :a, 11, & &1 + 1).private\n    %{a: 2}\n    iex> Req.Request.update_private(req, :b, 11, & &1 + 1).private\n    %{a: 1, b: 11}","ref":"Req.Request.html#update_private/4-examples"},{"type":"type","title":"Req.Request.error_step/0","doc":"An error step is a function that takes a request/exception tuple and returns a request/response\nor a request/exception tuple.\n\nThe function can be an anonymous function, or a `{module, function, args}` tuple. In the latter\ncase, the step is invoked as `apply(module, function, [request | args])`.\n\nSee also the [\"Response and Error Steps\"](#module-response-and-error-steps) section in the\nmodule documentation.","ref":"Req.Request.html#t:error_step/0"},{"type":"type","title":"Req.Request.request_step/0","doc":"A request step is a function that takes a request and returns a request or a tuple of request\nand response/exception.\n\nThe function can be an anonymous function, or a `{module, function, args}` tuple. In the latter\ncase, the step is invoked as `apply(module, function, [request | args])`.\n\nSee also the [\"Request Steps\"](#module-request-steps) section in the module documentation.","ref":"Req.Request.html#t:request_step/0"},{"type":"type","title":"Req.Request.response_step/0","doc":"A response step is a function that takes a request/response tuple and returns a request/response\nor a request/exception tuple.\n\nThe function can be an anonymous function, or a `{module, function, args}` tuple. In the latter\ncase, the step is invoked as `apply(module, function, [request | args])`.\n\nSee also the [\"Response and Error Steps\"](#module-response-and-error-steps) section in the\nmodule documentation.","ref":"Req.Request.html#t:response_step/0"},{"type":"type","title":"Req.Request.t/0","doc":"The request struct.","ref":"Req.Request.html#t:t/0"},{"type":"module","title":"Req.Response","doc":"The response struct.\n\nFields:\n\n  * `:status` - the HTTP status code.\n\n  * `:headers` - the HTTP response headers. The header names should be downcased.\n    See also \"Header Names\" section in `Req` module documentation.\n\n  * `:body` - the HTTP response body.\n\n  * `:trailers` - the HTTP response trailers. The trailer names must be downcased.\n\n  * `:private` - a map reserved for libraries and frameworks to use.\n    Prefix the keys with the name of your project to avoid any future\n    conflicts. Only accepts `t:atom/0` keys.","ref":"Req.Response.html"},{"type":"function","title":"Req.Response.delete_header/2","doc":"Deletes the header given by `name`.\n\nAll occurrences of the header are deleted, in case the header is repeated multiple times.\n\nSee also \"Header Names\" section in `Req` module documentation.","ref":"Req.Response.html#delete_header/2"},{"type":"function","title":"Examples - Req.Response.delete_header/2","doc":"iex> Req.Response.get_header(resp, \"cache-control\")\n    [\"max-age=600\", \"no-transform\"]\n    iex> resp = Req.Response.delete_header(resp, \"cache-control\")\n    iex> Req.Response.get_header(resp, \"cache-control\")\n    []","ref":"Req.Response.html#delete_header/2-examples"},{"type":"function","title":"Req.Response.get_header/2","doc":"Returns the values of the header specified by `name`.\n\nSee also \"Header Names\" section in `Req` module documentation.","ref":"Req.Response.html#get_header/2"},{"type":"function","title":"Examples - Req.Response.get_header/2","doc":"iex> Req.Response.get_header(response, \"content-type\")\n    [\"application/json\"]","ref":"Req.Response.html#get_header/2-examples"},{"type":"function","title":"Req.Response.get_private/3","doc":"Gets the value for a specific private `key`.","ref":"Req.Response.html#get_private/3"},{"type":"function","title":"Req.Response.get_retry_after/1","doc":"Returns the `retry-after` header delay value or nil if not found.","ref":"Req.Response.html#get_retry_after/1"},{"type":"function","title":"Req.Response.json/2","doc":"Builds or updates a response with JSON body.","ref":"Req.Response.html#json/2"},{"type":"function","title":"Example - Req.Response.json/2","doc":"iex> Req.Response.json(%{hello: 42})\n    %Req.Response{\n      status: 200,\n      headers: %{\"content-type\" => [\"application/json\"]},\n      body: ~s|{\"hello\":42}|\n    }\n\n    iex> resp = Req.Response.new()\n    iex> Req.Response.json(resp, %{hello: 42})\n    %Req.Response{\n      status: 200,\n      headers: %{\"content-type\" => [\"application/json\"]},\n      body: ~s|{\"hello\":42}|\n    }\n\nIf the request already contains a 'content-type' header, it is kept as is:\n\n    iex> Req.Response.new()\n    iex> |> Req.Response.put_header(\"content-type\", \"application/vnd.api+json; charset=utf-8\")\n    iex> |> Req.Response.json(%{hello: 42})\n    %Req.Response{\n      status: 200,\n      headers: %{\"content-type\" => [\"application/vnd.api+json; charset=utf-8\"]},\n      body: ~s|{\"hello\":42}|\n    }","ref":"Req.Response.html#json/2-example"},{"type":"function","title":"Req.Response.new/1","doc":"Returns a new response.\n\nExpects a keyword list, map, or struct containing the response keys.","ref":"Req.Response.html#new/1"},{"type":"function","title":"Example - Req.Response.new/1","doc":"iex> Req.Response.new(status: 200, body: \"body\")\n    %Req.Response{status: 200, headers: %{}, body: \"body\"}\n\n    iex> finch_response = %Finch.Response{status: 200, headers: [{\"content-type\", \"text/html\"}]}\n    iex> Req.Response.new(finch_response)\n    %Req.Response{status: 200, headers: %{\"content-type\" => [\"text/html\"]}, body: \"\"}","ref":"Req.Response.html#new/1-example"},{"type":"function","title":"Req.Response.put_header/3","doc":"Adds a new response header `name` if not present, otherwise replaces the\nprevious value of that header with `value`.\n\nSee also \"Header Names\" section in `Req` module documentation.","ref":"Req.Response.html#put_header/3"},{"type":"function","title":"Examples - Req.Response.put_header/3","doc":"iex> resp = Req.Response.put_header(%Req.Response{}, \"content-type\", \"application/json\")\n    iex> resp.headers\n    %{\"content-type\" => [\"application/json\"]}","ref":"Req.Response.html#put_header/3-examples"},{"type":"function","title":"Req.Response.put_private/3","doc":"Assigns a private `key` to `value`.","ref":"Req.Response.html#put_private/3"},{"type":"function","title":"Req.Response.update_private/4","doc":"Updates private `key` with the given function.\n\nIf `key` is present in request private map then the existing value is passed to `fun` and its\nresult is used as the updated value of `key`. If `key` is not present, `default` is inserted\nas the value of `key`. The default value will not be passed through the update function.","ref":"Req.Response.html#update_private/4"},{"type":"function","title":"Examples - Req.Response.update_private/4","doc":"iex> resp = %Req.Response{private: %{a: 1}}\n    iex> Req.Response.update_private(resp, :a, 11, & &1 + 1).private\n    %{a: 2}\n    iex> Req.Response.update_private(resp, :b, 11, & &1 + 1).private\n    %{a: 1, b: 11}","ref":"Req.Response.html#update_private/4-examples"},{"type":"type","title":"Req.Response.t/0","doc":"","ref":"Req.Response.html#t:t/0"},{"type":"module","title":"Req.Response.Async","doc":"Asynchronous response body.\n\nThis is the `response.body` when making a request with `into: :self`, that is,\nstreaming response body chunks to the current process mailbox.\n\nThis struct implements the `Enumerable` protocol where each element is a body chunk received\nfrom the current process mailbox. HTTP Trailer fields are ignored.\n\nIf the request is sent using HTTP/1, an extra process is spawned to consume messages from the\nunderlying socket. On both HTTP/1 and HTTP/2 the messages are sent to the current process as\nsoon as they arrive, as a firehose. If you wish to maximize request rate or have more control\nover how messages are streamed, use `into: fun` or `into: collectable` instead.\n\n**Note:** This feature is currently experimental and it may change in future releases.","ref":"Req.Response.Async.html"},{"type":"module","title":"Examples - Req.Response.Async","doc":"iex> resp = Req.get!(\"https://reqbin.org/ndjson?delay=1000\", into: :self)\n    iex> resp.body\n    #Req.Response.Async<...>\n    iex> Enum.each(resp.body, &IO.puts/1)\n    # {\"id\":0}\n    # {\"id\":1}\n    # {\"id\":2}\n    :ok","ref":"Req.Response.Async.html#module-examples"},{"type":"module","title":"Req.Steps","doc":"The collection of built-in steps.\n\nReq is composed of:\n\n  * `Req` - the high-level API\n\n  * `Req.Request` - the low-level API and the request struct\n\n  * `Req.Steps` - the collection of built-in steps (you're here!)\n\n  * `Req.Test` - the testing conveniences","ref":"Req.Steps.html"},{"type":"function","title":"Req.Steps.auth/1","doc":"Sets request authentication.","ref":"Req.Steps.html#auth/1"},{"type":"function","title":"Request Options - Req.Steps.auth/1","doc":"* `:auth` - sets the `authorization` header:\n\n      * `string` - sets to this value;\n\n      * `{:basic, userinfo}` - uses Basic HTTP authentication;\n\n      * `{:bearer, token}` - uses Bearer HTTP authentication;\n\n      * `:netrc` - load credentials from `.netrc` at path specified in `NETRC` environment variable.\n        If `NETRC` is not set, load `.netrc` in user's home directory;\n\n      * `{:netrc, path}` - load credentials from `path`\n\n      * `fn -> {:bearer, \"eyJ0eXAi...\" } end` - a 0-arity function that returns one of the aforementioned types.","ref":"Req.Steps.html#auth/1-request-options"},{"type":"function","title":"Examples - Req.Steps.auth/1","doc":"iex> Req.get!(\"https://httpbin.org/basic-auth/foo/bar\", auth: {:basic, \"foo:foo\"}).status\n    401\n    iex> Req.get!(\"https://httpbin.org/basic-auth/foo/bar\", auth: {:basic, \"foo:bar\"}).status\n    200\n    iex> Req.get!(\"https://httpbin.org/basic-auth/foo/bar\", auth: fn -> {:basic, \"foo:bar\"} end).status\n    200\n\n    iex> Req.get!(\"https://httpbin.org/bearer\", auth: {:bearer, \"\"}).status\n    401\n    iex> Req.get!(\"https://httpbin.org/bearer\", auth: {:bearer, \"foo\"}).status\n    200\n    iex> Req.get!(\"https://httpbin.org/bearer\", auth: fn -> {:bearer, \"foo\"} end).status\n    200\n\n    iex> System.put_env(\"NETRC\", \"./test/my_netrc\")\n    iex> Req.get!(\"https://httpbin.org/basic-auth/foo/bar\", auth: :netrc).status\n    200\n\n    iex> Req.get!(\"https://httpbin.org/basic-auth/foo/bar\", auth: {:netrc, \"./test/my_netrc\"}).status\n    200\n    iex> Req.get!(\"https://httpbin.org/basic-auth/foo/bar\", auth: fn -> {:netrc, \"./test/my_netrc\"} end).status\n    200","ref":"Req.Steps.html#auth/1-examples"},{"type":"function","title":"Req.Steps.cache/1","doc":"Performs HTTP caching using `if-modified-since` header.\n\nOnly successful (200 OK) responses are cached.\n\nThis step also _prepends_ a response step that loads and writes the cache. Be careful when\n_prepending_ other response steps, make sure the cache is loaded/written as soon as possible.","ref":"Req.Steps.html#cache/1"},{"type":"function","title":"Options - Req.Steps.cache/1","doc":"* `:cache` - if `true`, performs simple caching using `if-modified-since` header. Defaults to `false`.\n\n  * `:cache_dir` - the directory to store the cache, defaults to ` /req`\n    (see: `:filename.basedir/3`)","ref":"Req.Steps.html#cache/1-options"},{"type":"function","title":"Examples - Req.Steps.cache/1","doc":"iex> url = \"https://elixir-lang.org\"\n    iex> response1 = Req.get!(url, cache: true)\n    iex> response2 = Req.get!(url, cache: true)\n    iex> response1 == response2\n    true","ref":"Req.Steps.html#cache/1-examples"},{"type":"function","title":"Req.Steps.checksum/1","doc":"Sets expected response body checksum.","ref":"Req.Steps.html#checksum/1"},{"type":"function","title":"Request Options - Req.Steps.checksum/1","doc":"* `:checksum` - if set, this is the expected response body checksum.\n\n    Can be one of:\n\n      * `\"md5:(...)\"`\n      * `\"sha1:(...)\"`\n      * `\"sha256:(...)\"`","ref":"Req.Steps.html#checksum/1-request-options"},{"type":"function","title":"Examples - Req.Steps.checksum/1","doc":"iex> resp = Req.get!(\"https://httpbin.org/json\", checksum: \"sha1:9274ffd9cf273d4a008750f44540c4c5d4c8227c\")\n    iex> resp.status\n    200\n\n    iex> Req.get!(\"https://httpbin.org/json\", checksum: \"sha1:bad\")\n    ** (Req.ChecksumMismatchError) checksum mismatch\n    expected: sha1:bad\n    actual:   sha1:9274ffd9cf273d4a008750f44540c4c5d4c8227c","ref":"Req.Steps.html#checksum/1-examples"},{"type":"function","title":"Req.Steps.compress_body/1","doc":"Compresses the request body.","ref":"Req.Steps.html#compress_body/1"},{"type":"function","title":"Request Options - Req.Steps.compress_body/1","doc":"* `:compress_body` - if set to `true`, compresses the request body using gzip.\n    Defaults to `false`.","ref":"Req.Steps.html#compress_body/1-request-options"},{"type":"function","title":"Req.Steps.compressed/1","doc":"Asks the server to return compressed response.\n\nSupported formats:\n\n  * `gzip`\n\n  * `br` (if [brotli] is installed)\n\n  * `zstd` (if [ezstd] is installed)","ref":"Req.Steps.html#compressed/1"},{"type":"function","title":"Request Options - Req.Steps.compressed/1","doc":"* `:compressed` - if set to `true`, sets the `accept-encoding` header with compression\n    algorithms that Req supports. Defaults to `true`.\n\n    When streaming response body (`into: fun | collectable`), `compressed` defaults to `false`.","ref":"Req.Steps.html#compressed/1-request-options"},{"type":"function","title":"Examples - Req.Steps.compressed/1","doc":"Req automatically decompresses response body (`decompress_body/1` step) so let's disable that by\npassing `raw: true`.\n\nBy default, we ask the server to send compressed response. Let's look at the headers and the raw\nbody. Notice the body starts with `<<31, 139>>` (`<<0x1F, 0x8B>>`), the \"magic bytes\" for gzip:\n\n    iex> response = Req.get!(\"https://elixir-lang.org\", raw: true)\n    iex> Req.Response.get_header(response, \"content-encoding\")\n    [\"gzip\"]\n    iex> response.body |> binary_part(0, 2)\n    <<31, 139>>\n\nNow, let's pass `compressed: false` and notice the raw body was not compressed:\n\n    iex> response = Req.get!(\"https://elixir-lang.org\", raw: true, compressed: false)\n    iex> response.body |> binary_part(0, 15)\n    \"<!DOCTYPE html>\"\n\nThe Brotli and Zstandard compression algorithms are also supported if the optional\npackages are installed:\n\n    Mix.install([\n      :req,\n      {:brotli, \"~> 0.3.0\"},\n      {:ezstd, \"~> 1.0\"}\n    ])\n\n    response = Req.get!(\"https://httpbin.org/anything\")\n    response.body[\"headers\"][\"Accept-Encoding\"]\n    #=> \"zstd, br, gzip\"\n\n[brotli]: https://hex.pm/packages/brotli\n[ezstd]: https://hex.pm/packages/ezstd","ref":"Req.Steps.html#compressed/1-examples"},{"type":"function","title":"Req.Steps.decode_body/1","doc":"Decodes response body based on the detected format.\n\nSupported formats:\n\n| Format       | Decoder                                                           |\n| ------------ | ----------------------------------------------------------------- |\n| `json`       | `Jason.decode/2`                                                  |\n| `tar`, `tgz` | `:erl_tar.extract/2`                                              |\n| `zip`        | `:zip.unzip/2`                                                    |\n| `gzip`       | `:zlib.gunzip/1`                                                  |\n| `zst`        | `:ezstd.decompress/1` (if [ezstd] is installed)                   |\n| `csv`        | `NimbleCSV.RFC4180.parse_string/2` (if [nimble_csv] is installed) |\n\nThe format is determined based on the `content-type` header of the response. For example,\nif the `content-type` is `application/json`, the response body is decoded as JSON. The built-in\ndecoders also understand format extensions, such as decoding as JSON for a content-type of\n`application/vnd.api+json`. To do this, Req falls back to `MIME.extensions/1`; check the\ndocumentation for that function for more information.\n\nThis step is disabled on response body streaming. If response body is not a binary, in other\nwords it has been transformed by another step, it is left as is.","ref":"Req.Steps.html#decode_body/1"},{"type":"function","title":"Request Options - Req.Steps.decode_body/1","doc":"* `:decode_body` - if set to `false`, disables automatic response body decoding.\n    Defaults to `true`.\n\n  * `:decode_json` - options to pass to `Jason.decode/2`, defaults to `[]`.\n\n  * `:raw` - if set to `true`, disables response body decoding. Defaults to `false`.","ref":"Req.Steps.html#decode_body/1-request-options"},{"type":"function","title":"Examples - Req.Steps.decode_body/1","doc":"Decode JSON:\n\n    iex> response = Req.get!(\"https://httpbin.org/json\")\n    ...> response.body[\"slideshow\"][\"title\"]\n    \"Sample Slide Show\"\n\nDecode gzip:\n\n    iex> response = Req.get!(\"https://httpbin.org/gzip\")\n    ...> response.body[\"gzipped\"]\n    true\n\n[nimble_csv]: https://hex.pm/packages/nimble_csv\n[ezstd]: https://hex.pm/packages/ezstd","ref":"Req.Steps.html#decode_body/1-examples"},{"type":"function","title":"Req.Steps.decompress_body/1","doc":"Decompresses the response body based on the `content-encoding` header.\n\nThis step is disabled on response body streaming. If response body is not a binary, in other\nwords it has been transformed by another step, it is left as is.\n\nSupported formats:\n\n| Format        | Decoder                                         |\n| ------------- | ----------------------------------------------- |\n| gzip, x-gzip  | `:zlib.gunzip/1`                                |\n| br            | `:brotli.decode/1` (if [brotli] is installed)   |\n| zstd          | `:ezstd.decompress/1` (if [ezstd] is installed) |\n| _other_       | Returns data as is                              |\n\nThis step updates the following headers to reflect the changes:\n\n  * `content-encoding` is removed\n  * `content-length` is removed","ref":"Req.Steps.html#decompress_body/1"},{"type":"function","title":"Options - Req.Steps.decompress_body/1","doc":"* `:raw` - if set to `true`, disables response body decompression. Defaults to `false`.","ref":"Req.Steps.html#decompress_body/1-options"},{"type":"function","title":"Examples - Req.Steps.decompress_body/1","doc":"iex> response = Req.get!(\"https://httpbin.org/gzip\")\n    iex> response.body[\"gzipped\"]\n    true\n\nIf the [brotli] package is installed, Brotli is also supported:\n\n    Mix.install([\n      :req,\n      {:brotli, \"~> 0.3.0\"}\n    ])\n\n    response = Req.get!(\"https://httpbin.org/brotli\")\n    Req.Response.get_header(response, \"content-encoding\")\n    #=> [\"br\"]\n    response.body[\"brotli\"]\n    #=> true\n\n[brotli]: http://hex.pm/packages/brotli\n[ezstd]: https://hex.pm/packages/ezstd","ref":"Req.Steps.html#decompress_body/1-examples"},{"type":"function","title":"Req.Steps.encode_body/1","doc":"Encodes the request body.","ref":"Req.Steps.html#encode_body/1"},{"type":"function","title":"Request Options - Req.Steps.encode_body/1","doc":"* `:form` - if set, encodes the request body as `application/x-www-form-urlencoded`\n    (using `URI.encode_query/1`).\n\n  * `:form_multipart` - if set, encodes the request body as `multipart/form-data`.\n\n    It accepts `name` / `value` pairs. `value` can be one of:\n\n      * integer (automatically encoded as string)\n\n      * iodata\n\n      * `File.Stream`\n\n      * `{value, options}` tuple. Supported options are `:filename` and `:content_type`\n\n  * `:json` - if set, encodes the request body as JSON (using `Jason.encode_to_iodata!/1`), sets\n    the `accept` header to `application/json`, and the `content-type` header to `application/json`.","ref":"Req.Steps.html#encode_body/1-request-options"},{"type":"function","title":"Examples - Req.Steps.encode_body/1","doc":"Encoding form (`application/x-www-form-urlencoded`):\n\n    iex> Req.post!(\"https://httpbin.org/anything\", form: [a: 1]).body[\"form\"]\n    %{\"a\" => \"1\"}\n\nEncoding form (`multipart/form-data`):\n\n    iex> fields = [a: 1, b: {\"2\", filename: \"b.txt\"}]\n    iex> resp = Req.post!(\"https://httpbin.org/anything\", form_multipart: fields)\n    iex> resp.body[\"form\"]\n    %{\"a\" => \"1\"}\n    iex> resp.body[\"files\"]\n    %{\"b\" => \"2\"}\n\nEncoding JSON:\n\n    iex> Req.post!(\"https://httpbin.org/post\", json: %{a: 2}).body[\"json\"]\n    %{\"a\" => 2}","ref":"Req.Steps.html#encode_body/1-examples"},{"type":"function","title":"Req.Steps.handle_http_errors/1","doc":"Handles HTTP 4xx/5xx error responses.","ref":"Req.Steps.html#handle_http_errors/1"},{"type":"function","title":"Request Options - Req.Steps.handle_http_errors/1","doc":"* `:http_errors` - how to handle HTTP 4xx/5xx error responses. Can be one of the following:\n\n    * `:return` (default) - return the response\n\n    * `:raise` - raise an error","ref":"Req.Steps.html#handle_http_errors/1-request-options"},{"type":"function","title":"Examples - Req.Steps.handle_http_errors/1","doc":"iex> Req.get!(\"https://httpbin.org/status/404\").status\n    404\n\n    iex> Req.get!(\"https://httpbin.org/status/404\", http_errors: :raise)\n    ** (RuntimeError) The requested URL returned error: 404\n    Response body: \"\"","ref":"Req.Steps.html#handle_http_errors/1-examples"},{"type":"function","title":"Req.Steps.put_aws_sigv4/1","doc":"Signs request with AWS Signature Version 4.","ref":"Req.Steps.html#put_aws_sigv4/1"},{"type":"function","title":"Request Options - Req.Steps.put_aws_sigv4/1","doc":"* `:aws_sigv4` - if set, the AWS options to sign request:\n\n      * `:access_key_id` - the AWS access key id.\n\n      * `:secret_access_key` - the AWS secret access key.\n\n      * `:token` - if set, the AWS security token, for example returned from AWS STS.\n\n      * `:service` - the AWS service. We try to automatically detect the service (e.g.\n        `s3.amazonaws.com` host sets service to `:s3`)\n\n      * `:region` - the AWS region. Defaults to `\"us-east-1\"`.\n\n      * `:datetime` - the request datetime, defaults to `DateTime.utc_now(:second)`.","ref":"Req.Steps.html#put_aws_sigv4/1-request-options"},{"type":"function","title":"Examples - Req.Steps.put_aws_sigv4/1","doc":"iex> req =\n    ...>   Req.new(\n    ...>     base_url: \"https://s3.amazonaws.com\",\n    ...>     aws_sigv4: [\n    ...>       access_key_id: System.get_env(\"AWS_ACCESS_KEY_ID\"),\n    ...>       secret_access_key: System.get_env(\"AWS_SECRET_ACCESS_KEY\")\n    ...>     ]\n    ...>   )\n    iex>\n    iex> %{status: 200} = Req.put!(req, url: \"/bucket1/key1\", body: \"Hello, World!\")\n    iex> resp = Req.get!(req, url: \"/bucket1/key1\").body\n    \"Hello, World!\"\n\nRequest body streaming also works though `content-length` header must be explicitly set:\n\n    iex> path = \"a.txt\"\n    iex> File.write!(path, String.duplicate(\"a\", 100_000))\n    iex> size = File.stat!(path).size\n    iex> chunk_size = 10 * 1024\n    iex> stream = File.stream!(path, chunk_size)\n    iex> %{status: 200} = Req.put!(req, url: \"/key1\", headers: [content_length: size], body: stream)\n    iex> byte_size(Req.get!(req, \"/bucket1/key1\").body)\n    100_000","ref":"Req.Steps.html#put_aws_sigv4/1-examples"},{"type":"function","title":"Req.Steps.put_base_url/1","doc":"Sets base URL for all requests.","ref":"Req.Steps.html#put_base_url/1"},{"type":"function","title":"Request Options - Req.Steps.put_base_url/1","doc":"* `:base_url` - if set, the request URL is merged with this base URL.\n\n    The base url can be a string, a `%URI{}` struct, a 0-arity function,\n    or a `{mod, fun, args}` tuple describing a function to call.","ref":"Req.Steps.html#put_base_url/1-request-options"},{"type":"function","title":"Examples - Req.Steps.put_base_url/1","doc":"iex> req = Req.new(base_url: \"https://httpbin.org\")\n    iex> Req.get!(req, url: \"/status/200\").status\n    200\n    iex> Req.get!(req, url: \"/status/201\").status\n    201","ref":"Req.Steps.html#put_base_url/1-examples"},{"type":"function","title":"Req.Steps.put_params/1","doc":"Adds params to request query string.","ref":"Req.Steps.html#put_params/1"},{"type":"function","title":"Request Options - Req.Steps.put_params/1","doc":"* `:params` - params to add to the request query string. Defaults to `[]`.","ref":"Req.Steps.html#put_params/1-request-options"},{"type":"function","title":"Examples - Req.Steps.put_params/1","doc":"iex> Req.get!(\"https://httpbin.org/anything/query\", params: [x: 1, y: 2]).body[\"args\"]\n    %{\"x\" => \"1\", \"y\" => \"2\"}","ref":"Req.Steps.html#put_params/1-examples"},{"type":"function","title":"Req.Steps.put_path_params/1","doc":"Uses a templated request path.\n\nBy default, params in the URL path are expressed as strings prefixed with `:`. For example,\n`:code` in `https://httpbin.org/status/:code`. If you want to use the `{code}` syntax,\nset `path_params_style: :curly`. Param names must start with a letter and can contain letters,\ndigits, and underscores; this is true both for `:colon_params` as well as `{curly_params}`.\n\nPath params are replaced in the request URL path. The path params are specified as a keyword\nlist of parameter names and values, as in the examples below. The values of the parameters are\nconverted to strings using the `String.Chars` protocol (`to_string/1`).","ref":"Req.Steps.html#put_path_params/1"},{"type":"function","title":"Request Options - Req.Steps.put_path_params/1","doc":"* `:path_params` - params to add to the templated path. Defaults to `[]`.\n\n  * `:path_params_style` (*available since v0.5.1*) - how path params are expressed. Can be one of:\n\n       * `:colon` - (default) for Plug-style parameters, such as `:code` in\n         `https://httpbin.org/status/:code`.\n\n       * `:curly` - for [OpenAPI](https://swagger.io/specification/)-style parameters, such as\n         `{code}` in `https://httpbin.org/status/{code}`.","ref":"Req.Steps.html#put_path_params/1-request-options"},{"type":"function","title":"Examples - Req.Steps.put_path_params/1","doc":"iex> Req.get!(\"https://httpbin.org/status/:code\", path_params: [code: 201]).status\n    201\n\n    iex> Req.get!(\"https://httpbin.org/status/{code}\", path_params: [code: 201], path_params_style: :curly).status\n    201","ref":"Req.Steps.html#put_path_params/1-examples"},{"type":"function","title":"Req.Steps.put_plug/1","doc":"Sets adapter to `run_plug/1`.\n\nSee `run_plug/1` for more information.","ref":"Req.Steps.html#put_plug/1"},{"type":"function","title":"Request Options - Req.Steps.put_plug/1","doc":"* `:plug` - if set, the plug to run the request through.","ref":"Req.Steps.html#put_plug/1-request-options"},{"type":"function","title":"Req.Steps.put_range/1","doc":"Sets the \"Range\" request header.","ref":"Req.Steps.html#put_range/1"},{"type":"function","title":"Request Options - Req.Steps.put_range/1","doc":"* `:range` - can be one of the following:\n\n      * a string - returned as is\n\n      * a `first..last` range - converted to `\"bytes= - \"`","ref":"Req.Steps.html#put_range/1-request-options"},{"type":"function","title":"Examples - Req.Steps.put_range/1","doc":"iex> response = Req.get!(\"https://httpbin.org/range/100\", range: 0..3)\n    iex> response.status\n    206\n    iex> response.body\n    \"abcd\"\n    iex> Req.Response.get_header(response, \"content-range\")\n    [\"bytes 0-3/100\"]","ref":"Req.Steps.html#put_range/1-examples"},{"type":"function","title":"Req.Steps.put_user_agent/1","doc":"Sets the user-agent header.","ref":"Req.Steps.html#put_user_agent/1"},{"type":"function","title":"Request Options - Req.Steps.put_user_agent/1","doc":"* `:user_agent` - sets the `user-agent` header. Defaults to `\"req/0.5.8\"`.","ref":"Req.Steps.html#put_user_agent/1-request-options"},{"type":"function","title":"Examples - Req.Steps.put_user_agent/1","doc":"iex> Req.get!(\"https://httpbin.org/user-agent\").body\n    %{\"user-agent\" => \"req/0.5.8\"}\n\n    iex> Req.get!(\"https://httpbin.org/user-agent\", user_agent: \"foo\").body\n    %{\"user-agent\" => \"foo\"}","ref":"Req.Steps.html#put_user_agent/1-examples"},{"type":"function","title":"Req.Steps.redirect/1","doc":"Follows redirects.\n\nThe original request method may be changed to GET depending on the status code:\n\n| Code          | Method handling    |\n| ------------- | ------------------ |\n| 301, 302, 303 | Changed to GET     |\n| 307, 308      | Method not changed |","ref":"Req.Steps.html#redirect/1"},{"type":"function","title":"Request Options - Req.Steps.redirect/1","doc":"* `:redirect` - if set to `false`, disables automatic response redirects.\n    Defaults to `true`.\n\n  * `:redirect_trusted` - by default, authorization credentials are only sent\n    on redirects with the same host, scheme and port. If `:redirect_trusted` is set\n    to `true`, credentials will be sent to any host.\n\n  * `:redirect_log_level` - the log level to emit redirect logs at. Can also be set\n    to `false` to disable logging these messages. Defaults to `:debug`.\n\n  * `:max_redirects` - the maximum number of redirects, defaults to `10`. If the\n    limit is reached, the pipeline is halted and a `Req.TooManyRedirectsError`\n    exception is returned.","ref":"Req.Steps.html#redirect/1-request-options"},{"type":"function","title":"Examples - Req.Steps.redirect/1","doc":"iex> Req.get!(\"http://api.github.com\").status\n    # 23:24:11.670 [debug] redirecting to https://api.github.com/\n    200\n\n    iex> Req.get!(\"https://httpbin.org/redirect/4\", max_redirects: 3)\n    # 23:07:59.570 [debug] redirecting to /relative-redirect/3\n    # 23:08:00.068 [debug] redirecting to /relative-redirect/2\n    # 23:08:00.206 [debug] redirecting to /relative-redirect/1\n    ** (RuntimeError) too many redirects (3)\n\n    iex> Req.get!(\"http://api.github.com\", redirect_log_level: false)\n    200\n\n    iex> Req.get!(\"http://api.github.com\", redirect_log_level: :error)\n    # 23:24:11.670 [error]  redirecting to https://api.github.com/\n    200","ref":"Req.Steps.html#redirect/1-examples"},{"type":"function","title":"Req.Steps.retry/1","doc":"Retries a request in face of errors.\n\nThis function can be used as either or both response and error step.","ref":"Req.Steps.html#retry/1"},{"type":"function","title":"Request Options - Req.Steps.retry/1","doc":"* `:retry` - can be one of the following:\n\n      * `:safe_transient` (default) - retry safe (GET/HEAD) requests on one of:\n\n          * HTTP 408/429/500/502/503/504 responses\n\n          * `Req.TransportError` with `reason: :timeout | :econnrefused | :closed`\n\n          * `Req.HTTPError` with `protocol: :http2, reason: :unprocessed`\n\n      * `:transient` - same as `:safe_transient` except retries all HTTP methods (POST, DELETE, etc.)\n\n      * `fun` - a 2-arity function that accepts a `Req.Request` and either a `Req.Response` or an exception struct\n        and returns one of the following:\n\n          * `true` - retry with the default delay controller by default delay option described below.\n\n          * `{:delay, milliseconds}` - retry with the given delay.\n\n          * `false/nil` - don't retry.\n\n      * `false` - don't retry.\n\n  * `:retry_delay` - if not set, which is the default, the retry delay is determined by\n    the value of the `Retry-After` header on HTTP 429/503 responses. If the header is not set,\n    the default delay follows a simple exponential backoff: 1s, 2s, 4s, 8s, ...\n\n    `:retry_delay` can be set to a function that receives the retry count (starting at 0)\n    and returns the delay, the number of milliseconds to sleep before making another attempt.\n\n  * `:retry_log_level` - the log level to emit retry logs at. Can also be set to `false` to disable\n    logging these messages. Defaults to `:warning`.\n\n  * `:max_retries` - maximum number of retry attempts, defaults to `3` (for a total of `4`\n    requests to the server, including the initial one.)","ref":"Req.Steps.html#retry/1-request-options"},{"type":"function","title":"Examples - Req.Steps.retry/1","doc":"With default options:\n\n    iex> Req.get!(\"https://httpbin.org/status/500,200\").status\n    # 19:02:08.463 [warning] retry: got response with status 500, will retry in 2000ms, 2 attempts left\n    # 19:02:10.710 [warning] retry: got response with status 500, will retry in 4000ms, 1 attempt left\n    200\n\nDelay with jitter:\n\n    iex> delay = fn n -> trunc(Integer.pow(2, n) * 1000 * (1 - 0.1 * :rand.uniform())) end\n    iex> Req.get!(\"https://httpbin.org/status/500,200\", retry_delay: delay).status\n    # 08:43:19.101 [warning] retry: got response with status 500, will retry in 941ms, 2 attempts left\n    # 08:43:22.958 [warning] retry: got response with status 500, will retry in 1877s, 1 attempt left\n    200","ref":"Req.Steps.html#retry/1-examples"},{"type":"function","title":"Req.Steps.run_finch/1","doc":"Runs the request using `Finch`.\n\nThis is the default Req _adapter_. See\n[\"Adapter\" section in the `Req.Request`](Req.Request.html#module-adapter) module documentation\nfor more information on adapters.\n\nFinch returns `Mint.TransportError` exceptions on HTTP connection problems. These are automatically\nconverted to `Req.TransportError` exceptions. Similarly, HTTP-protocol-related errors,\n`Mint.HTTPError` and `Finch.Error`, and converted to `Req.HTTPError`.","ref":"Req.Steps.html#run_finch/1"},{"type":"function","title":"HTTP/1 Pools - Req.Steps.run_finch/1","doc":"On HTTP/1 connections, Finch creates a pool per `{scheme, host, port}` tuple. These pools\nare kept around to re-use connections as much as possible, however they are **not automatically\nterminated**. To do so, you can configure custom Finch pool:\n\n    {:ok, _} =\n      Finch.start_link(\n        name: MyFinch,\n        pools: %{\n          default: [\n            # terminate idle {scheme, host, port} pool after 60s\n            pool_max_idle_time: 60_000\n          ]\n        }\n      )\n\n    Req.get!(\"https://httpbin.org/json\", finch: MyFinch)","ref":"Req.Steps.html#run_finch/1-http-1-pools"},{"type":"function","title":"Request Options - Req.Steps.run_finch/1","doc":"* `:finch` - the name of the Finch pool. Defaults to a pool automatically started by Req.\n\n  * `:connect_options` - dynamically starts (or re-uses already started) Finch pool with\n    the given connection options:\n\n      * `:timeout` - socket connect timeout in milliseconds, defaults to `30_000`.\n\n      * `:protocols` - the HTTP protocols to use, defaults to\n        `[:http1]`.\n\n      * `:hostname` - Mint explicit hostname, see `Mint.HTTP.connect/4` for more information.\n\n      * `:transport_opts` - Mint transport options, see `Mint.HTTP.connect/4` for more\n      information.\n\n      * `:proxy_headers` - Mint proxy headers, see `Mint.HTTP.connect/4` for more information.\n\n      * `:proxy` - Mint HTTP/1 proxy settings, a `{schema, address, port, options}` tuple.\n        See `Mint.HTTP.connect/4` for more information.\n\n      * `:client_settings` - Mint HTTP/2 client settings, see `Mint.HTTP.connect/4` for more\n      information.\n\n  * `:inet6` - if set to true, uses IPv6.\n\n    If the request URL looks like IPv6 address, i.e., say, `[::1]`, it defaults to `true`\n    and otherwise defaults to `false`.\n    This is a shortcut for setting `connect_options: [transport_opts: [inet6: true]]`.\n\n  * `:pool_timeout` - pool checkout timeout in milliseconds, defaults to `5000`.\n\n  * `:receive_timeout` - socket receive timeout in milliseconds, defaults to `15_000`.\n\n  * `:unix_socket` - if set, connect through the given UNIX domain socket.\n\n  * `:pool_max_idle_time` - the maximum number of milliseconds that a pool can be\n    idle before being terminated, used only by HTTP1 pools. Default to `:infinity`.\n\n  * `:finch_private` - a map or keyword list of private metadata to add to the Finch request.\n    May be useful for adding custom data when handling telemetry with `Finch.Telemetry`.\n\n  * `:finch_request` - a function that executes the Finch request, defaults to using\n    `Finch.request/3`.\n\n    The function should accept 4 arguments:\n\n      * `request` - the `%Req.Request{}` struct\n\n      * `finch_request` - the Finch request\n\n      * `finch_name` - the Finch name\n\n      * `finch_options` - the Finch options\n\n    And it should return either `{request, response}` or `{request, exception}`.","ref":"Req.Steps.html#run_finch/1-request-options"},{"type":"function","title":"Examples - Req.Steps.run_finch/1","doc":"Custom `:receive_timeout`:\n\n    iex> Req.get!(url, receive_timeout: 1000)\n\nConnecting through UNIX socket:\n\n    iex> Req.get!(\"http:///v1.41/_ping\", unix_socket: \"/var/run/docker.sock\").body\n    \"OK\"\n\nCustom connection options:\n\n    iex> Req.get!(url, connect_options: [timeout: 5000])\n\n    iex> Req.get!(url, connect_options: [protocols: [:http2]])\n\nConnecting without certificate check (useful in development, not recommended in production):\n\n    iex> Req.get!(url, connect_options: [transport_opts: [verify: :verify_none]])\n\nConnecting with custom certificates:\n\n    iex> Req.get!(url, connect_options: [transport_opts: [cacertfile: \"certs.pem\"]])\n\nConnecting through a proxy with basic authentication:\n\n    iex> Req.new(\n    ...>  url: \"https://elixir-lang.org\",\n    ...>  connect_options: [\n    ...>    proxy: {:http, \"your.proxy.com\", 8888, []},\n    ...>    proxy_headers: [{\"proxy-authorization\", \"Basic \" <> Base.encode64(\"user:pass\")}]\n    ...>  ]\n    ...> )\n    iex> |> Req.get!()\n\nTransport errors are represented as `Req.TransportError` exceptions:\n\n    iex> Req.get(\"https://httpbin.org/delay/1\", receive_timeout: 0, retry: false)\n    {:error, %Req.TransportError{reason: :timeout}}","ref":"Req.Steps.html#run_finch/1-examples"},{"type":"function","title":"Req.Steps.run_plug/1","doc":"Runs the request against a plug instead of over the network.\n\nThis step is a Req _adapter_. It is set as the adapter by the `put_plug/1` step\nif the `:plug` option is set.\n\nIt requires [`:plug`](https://hexdocs.pm/plug) dependency:\n\n    {:plug, \"~> 1.0\"}","ref":"Req.Steps.html#run_plug/1"},{"type":"function","title":"Request Options - Req.Steps.run_plug/1","doc":"* `:plug` - the plug to run the request through. It can be one of:\n\n      * A _function_ plug: a `fun(conn)` or `fun(conn, options)` function that takes a\n        `Plug.Conn` and returns a `Plug.Conn`.\n\n      * A _module_ plug: a `module` name or a `{module, options}` tuple.\n\n    Req automatically calls `Plug.Conn.fetch_query_params/2` before your plug, so you can\n    get query params using `conn.query_params`.","ref":"Req.Steps.html#run_plug/1-request-options"},{"type":"function","title":"Examples - Req.Steps.run_plug/1","doc":"This step is particularly useful to test plugs:\n\n    defmodule Echo do\n      def call(conn, _) do\n        \"/\" <> path = conn.request_path\n        Plug.Conn.send_resp(conn, 200, path)\n      end\n    end\n\n    test \"echo\" do\n      assert Req.get!(\"http:///hello\", plug: Echo).body == \"hello\"\n    end\n\nYou can define plugs as functions too:\n\n    test \"echo\" do\n      echo = fn conn ->\n        \"/\" <> path = conn.request_path\n        Plug.Conn.send_resp(conn, 200, path)\n      end\n\n      assert Req.get!(\"http:///hello\", plug: echo).body == \"hello\"\n    end\n\nwhich is particularly useful to create HTTP service stubs, similar to tools like\n[Bypass](https://github.com/PSPDFKit-labs/bypass).\n\nResponse streaming is also supported however at the moment the entire response\nbody is emitted as one chunk:\n\n    test \"echo\" do\n      plug = fn conn ->\n        conn = Plug.Conn.send_chunked(conn, 200)\n        {:ok, conn} = Plug.Conn.chunk(conn, \"echo\")\n        {:ok, conn} = Plug.Conn.chunk(conn, \"echo\")\n        conn\n      end\n\n      assert Req.get!(plug: plug, into: []).body == [\"echoecho\"]\n    end\n\nWhen testing JSON APIs, it's common to use the `Req.Test.json/2` helper:\n\n    test \"JSON\" do\n      plug = fn conn ->\n        Req.Test.json(conn, %{message: \"Hello, World!\"})\n      end\n\n      resp = Req.get!(plug: plug)\n      assert resp.status == 200\n      assert resp.headers[\"content-type\"] == [\"application/json; charset=utf-8\"]\n      assert resp.body == %{\"message\" => \"Hello, World!\"}\n    end\n\nYou can simulate network errors by calling `Req.Test.transport_error/2`\nin your plugs:\n\n    test \"network issues\" do\n      plug = fn conn ->\n        Req.Test.transport_error(conn, :timeout)\n      end\n\n      assert Req.get(plug: plug, retry: false) ==\n               {:error, %Req.TransportError{reason: :timeout}}\n    end","ref":"Req.Steps.html#run_plug/1-examples"},{"type":"function","title":"Req.Steps.verify_checksum/1","doc":"Verifies the response body checksum.\n\nSee `checksum/1` for more information.","ref":"Req.Steps.html#verify_checksum/1"},{"type":"module","title":"Req.Test","doc":"Req testing conveniences.\n\nReq is composed of:\n\n  * `Req` - the high-level API\n\n  * `Req.Request` - the low-level API and the request struct\n\n  * `Req.Steps` - the collection of built-in steps\n\n  * `Req.Test` - the testing conveniences (you're here!)\n\nReq already has built-in support for different variants of stubs via `:plug`, `:adapter`,\nand (indirectly) `:base_url` options. With this module you can:\n\n  * Create request stubs using [`Req.Test.stub(name, plug)`](`stub/2`) and mocks\n    using [`Req.Test.expect(name, count, plug)`](`expect/3`). Both can be used in concurrent\n    tests.\n\n  * Configure Req to run requests through mocks/stubs by setting `plug: {Req.Test, name}`.\n    This works because `Req.Test` itself is a plug whose job is to fetch the mocks/stubs under\n    `name`.\n\n  * Easily create JSON responses with [`Req.Test.json(conn, body)`](`json/2`),\n    HTML responses with [`Req.Test.html(conn, body)`](`html/2`), and\n    text responses with [`Req.Test.text(conn, body)`](`text/2`).\n\n  * Simulate network errors with [`Req.Test.transport_error(conn, reason)`](`transport_error/2`).\n\nMocks and stubs are using the same ownership model of\n[nimble_ownership](https://hex.pm/packages/nimble_ownership), also used by\n[Mox](https://hex.pm/packages/mox). This allows `Req.Test` to be used in concurrent tests.","ref":"Req.Test.html"},{"type":"module","title":"Example - Req.Test","doc":"Imagine we're building an app that displays weather for a given location using an HTTP weather\nservice:\n\n    defmodule MyApp.Weather do\n      def get_rating(location) do\n        case get_temperature(location) do\n          {:ok, %{status: 200, body: %{\"celsius\" => celsius}}} ->\n            cond do\n              celsius < 18.0 -> {:ok, :too_cold}\n              celsius < 30.0 -> {:ok, :nice}\n              true -> {:ok, :too_hot}\n            end\n\n          _ ->\n            :error\n        end\n      end\n\n      def get_temperature(location) do\n        [\n          base_url: \"https://weather-service\"\n        ]\n        |> Keyword.merge(Application.get_env(:myapp, :weather_req_options, []))\n        |> Req.request()\n      end\n    end\n\nWe configure it for production:\n\n    # config/runtime.exs\n    config :myapp, weather_req_options: [\n      auth: {:bearer, System.fetch_env!(\"MYAPP_WEATHER_API_KEY\")}\n    ]\n\nIn tests, instead of hitting the network, we make the request against\na [plug](`Req.Steps.run_plug/1`) _stub_ named `MyApp.Weather`:\n\n    # config/test.exs\n    config :myapp, weather_req_options: [\n      plug: {Req.Test, MyApp.Weather}\n    ]\n\nNow we can control our stubs **in concurrent tests**:\n\n    use ExUnit.Case, async: true\n\n    test \"nice weather\" do\n      Req.Test.stub(MyApp.Weather, fn conn ->\n        Req.Test.json(conn, %{\"celsius\" => 25.0})\n      end)\n\n      assert MyApp.Weather.get_rating(\"Krakow, Poland\") == {:ok, :nice}\n    end","ref":"Req.Test.html#module-example"},{"type":"module","title":"Concurrency and Allowances - Req.Test","doc":"The example above works in concurrent tests because `MyApp.Weather.get_rating/1` calls\ndirectly to `Req.request/1` *in the same process*. It also works in many cases where the\nrequest happens in a spawned process, such as a `Task`, `GenServer`, and more.\n\nHowever, if you are encountering issues with stubs not being available in spawned processes,\nit's likely that you'll need **explicit allowances**. For example, if\n`MyApp.Weather.get_rating/1` was calling `Req.request/1` in a process spawned with `spawn/1`,\nthe stub would not be available in the spawned process:\n\n    # With code like this, the stub would not be available in the spawned task:\n    def get_rating_async(location) do\n      spawn(fn -> get_rating(location) end)\n    end\n\nTo make stubs defined in the test process available in other processes, you can use\n`allow/3`. For example, imagine that the call to `MyApp.Weather.get_rating/1`\nwas happening in a spawned GenServer:\n\n    test \"nice weather\" do\n      {:ok, pid} = start_gen_server(...)\n\n      Req.Test.stub(MyApp.Weather, fn conn ->\n        Req.Test.json(conn, %{\"celsius\" => 25.0})\n      end)\n\n      Req.Test.allow(MyApp.Weather, self(), pid)\n\n      assert get_weather(pid, \"Krakow, Poland\") == {:ok, :nice}\n    end","ref":"Req.Test.html#module-concurrency-and-allowances"},{"type":"module","title":"Broadway - Req.Test","doc":"If you're using `Req.Test` with [Broadway](https://hex.pm/broadway), you may need to use\n`allow/3` to make stubs available in the Broadway processors. A great way to do that is\nto hook into the [Telemetry](https://hex.pm/telemetry) events that Broadway publishes to\nmanually allow the processors and batch processors to access the stubs. This approach is\nsimilar to what is [documented in Broadway\nitself](https://hexdocs.pm/broadway/Broadway.html#module-testing-with-ecto).\n\nFirst, you should add the test PID (which is allowed to use the Req stub) to the metadata\nfor the test events you're publishing:\n\n    Broadway.test_message(MyApp.Pipeline, message, metadata: %{req_stub_owner: self()})\n\nThen, you'll need to define a test helper to hook into the Telemetry events. For example,\nin your `test/test_helper.exs` file:\n\n    defmodule BroadwayReqStubs do\n      def attach(stub) do\n        events = [\n          [:broadway, :processor, :start],\n          [:broadway, :batch_processor, :start],\n        ]\n\n        :telemetry.attach_many({__MODULE__, stub}, events, &__MODULE__.handle_event/4, %{stub: stub})\n      end\n\n      def handle_event(_event_name, _event_measurement, %{messages: messages}, %{stub: stub}) do\n        with [%Broadway.Message{metadata: %{req_stub_owner: pid}} | _] <- messages do\n          :ok = Req.Test.allow(stub, pid, self())\n        end\n\n        :ok\n      end\n    end\n\nLast but not least, attach the helper in your `test/test_helper.exs`:\n\n    BroadwayReqStubs.attach(MyStub)","ref":"Req.Test.html#module-broadway"},{"type":"function","title":"Req.Test.allow/3","doc":"Allows `pid_to_allow` to access `name` provided that `owner` is already allowed.","ref":"Req.Test.html#allow/3"},{"type":"function","title":"Req.Test.expect/3","doc":"Creates a request expectation with the given `name` and `plug`, expected to be fetched at\nmost `n` times, **in order**.\n\nThis function allows you to expect a `n` number of request and handle them **in order** via the\ngiven `plug`. It is safe to use in concurrent tests. If you fetch the value under `name` more\nthan `n` times, this function raises a `RuntimeError`.\n\nThe `name` can be any term.\n\nThe `plug` can be one of:\n\n  * A _function_ plug: a `fun(conn)` or `fun(conn, options)` function that takes a\n    `Plug.Conn` and returns a `Plug.Conn`.\n\n  * A _module_ plug: a `module` name or a `{module, options}` tuple.\n\nSee `stub/2` and module documentation for more information.","ref":"Req.Test.html#expect/3"},{"type":"function","title":"Examples - Req.Test.expect/3","doc":"Let's simulate a server that is having issues: on the first request it is not responding\nand on the following two requests it returns an HTTP 500. Only on the third request it returns\nan HTTP 200. Req by default automatically retries transient errors (using `Req.Steps.retry/1`)\nso it will make multiple requests exercising all of our request expectations:\n\n    iex> Req.Test.expect(MyStub, &Req.Test.transport_error(&1, :econnrefused))\n    iex> Req.Test.expect(MyStub, 2, &Plug.Conn.send_resp(&1, 500, \"internal server error\"))\n    iex> Req.Test.expect(MyStub, &Plug.Conn.send_resp(&1, 200, \"ok\"))\n    iex> Req.get!(plug: {Req.Test, MyStub}).body\n    # 15:57:06.309 [warning] retry: got exception, will retry in 1000ms, 3 attempts left\n    # 15:57:06.309 [warning] ** (Req.TransportError) connection refused\n    # 15:57:07.310 [warning] retry: got response with status 500, will retry in 2000ms, 2 attempts left\n    # 15:57:09.311 [warning] retry: got response with status 500, will retry in 4000ms, 1 attempt left\n    \"ok\"\n\n    iex> Req.request!(plug: {Req.Test, MyStub})\n    ** (RuntimeError) no mock or stub for MyStub","ref":"Req.Test.html#expect/3-examples"},{"type":"function","title":"Req.Test.html/2","doc":"Sends HTML response.","ref":"Req.Test.html#html/2"},{"type":"function","title":"Examples - Req.Test.html/2","doc":"iex> plug = fn conn ->\n    ...>   Req.Test.html(conn, \" Hello, World! \")\n    ...> end\n    iex>\n    iex> resp = Req.get!(plug: plug)\n    iex> resp.headers[\"content-type\"]\n    [\"text/html; charset=utf-8\"]\n    iex> resp.body\n    \" Hello, World! \"","ref":"Req.Test.html#html/2-examples"},{"type":"function","title":"Req.Test.json/2","doc":"Sends JSON response.","ref":"Req.Test.html#json/2"},{"type":"function","title":"Examples - Req.Test.json/2","doc":"iex> plug = fn conn ->\n    ...>   Req.Test.json(conn, %{celsius: 25.0})\n    ...> end\n    iex>\n    iex> resp = Req.get!(plug: plug)\n    iex> resp.headers[\"content-type\"]\n    [\"application/json; charset=utf-8\"]\n    iex> resp.body\n    %{\"celsius\" => 25.0}","ref":"Req.Test.html#json/2-examples"},{"type":"function","title":"Req.Test.set_req_test_from_context/1","doc":"Sets the `Req.Test` mode based on the given `ExUnit` context.\n\nThis works as a ExUnit callback:\n\n    setup :set_req_test_from_context","ref":"Req.Test.html#set_req_test_from_context/1"},{"type":"function","title":"Req.Test.set_req_test_to_private/1","doc":"Sets the `Req.Test` mode to \"private\", meaning that stubs can be shared across\ntests concurrently.","ref":"Req.Test.html#set_req_test_to_private/1"},{"type":"function","title":"Req.Test.set_req_test_to_shared/1","doc":"Sets the `Req.Test` mode to \"global\", meaning that the stubs are shared across all tests\nand cannot be used concurrently.","ref":"Req.Test.html#set_req_test_to_shared/1"},{"type":"function","title":"Req.Test.stub/2","doc":"Creates a request stub with the given `name` and `plug`.\n\nReq allows running requests against _plugs_ (instead of over the network) using the\n[`:plug`](`Req.Steps.run_plug/1`) option. However, passing the `:plug` value throughout the\nsystem can be cumbersome. Instead, you can tell Req to find plugs by `name` by setting\n`plug: {Req.Test, name}`, and register plug stubs for that `name` by calling\n`Req.Test.stub(name, plug)`. In other words, multiple concurrent tests can register test stubs\nunder the same `name`, and when Req makes the request, it will find the appropriate\nimplementation, even when invoked from different processes than the test process.\n\nThe `name` can be any term.\n\nThe `plug` can be one of:\n\n  * A _function_ plug: a `fun(conn)` or `fun(conn, options)` function that takes a\n    `Plug.Conn` and returns a `Plug.Conn`.\n\n  * A _module_ plug: a `module` name or a `{module, options}` tuple.","ref":"Req.Test.html#stub/2"},{"type":"function","title":"Examples - Req.Test.stub/2","doc":"iex> Req.Test.stub(MyStub, fn conn ->\n    ...>   send(self(), :req_happened)\n    ...>   Req.Test.json(conn, %{})\n    ...> end)\n    :ok\n    iex> Req.get!(plug: {Req.Test, MyStub}).body\n    %{}\n    iex> receive do\n    ...>   :req_happened -> :ok\n    ...> end\n    :ok","ref":"Req.Test.html#stub/2-examples"},{"type":"function","title":"Req.Test.text/2","doc":"Sends text response.","ref":"Req.Test.html#text/2"},{"type":"function","title":"Examples - Req.Test.text/2","doc":"iex> plug = fn conn ->\n    ...>   Req.Test.text(conn, \"Hello, World!\")\n    ...> end\n    iex>\n    iex> resp = Req.get!(plug: plug)\n    iex> resp.headers[\"content-type\"]\n    [\"text/plain; charset=utf-8\"]\n    iex> resp.body\n    \"Hello, World!\"","ref":"Req.Test.html#text/2-examples"},{"type":"function","title":"Req.Test.transport_error/2","doc":"Simulates a network transport error.","ref":"Req.Test.html#transport_error/2"},{"type":"function","title":"Examples - Req.Test.transport_error/2","doc":"iex> plug = fn conn ->\n    ...>   Req.Test.transport_error(conn, :timeout)\n    ...> end\n    iex>\n    iex> Req.get(plug: plug, retry: false)\n    {:error, %Req.TransportError{reason: :timeout}}","ref":"Req.Test.html#transport_error/2-examples"},{"type":"function","title":"Req.Test.verify!/0","doc":"Verifies that all the plugs expected to be executed within any scope have been executed.","ref":"Req.Test.html#verify!/0"},{"type":"function","title":"Req.Test.verify!/1","doc":"Verifies that all the plugs expected to be executed within the scope of `name` have been\nexecuted.","ref":"Req.Test.html#verify!/1"},{"type":"function","title":"Req.Test.verify_on_exit!/1","doc":"Sets a ExUnit callback to verify the expectations on exit.\n\nSimilar to calling `verify!/0` at the end of your test.","ref":"Req.Test.html#verify_on_exit!/1"},{"type":"exception","title":"Req.ArchiveError","doc":"Represents an error when unpacking archives fails, returned by `Req.Steps.decode_body/1`.","ref":"Req.ArchiveError.html"},{"type":"exception","title":"Req.ChecksumMismatchError","doc":"Represents a checksum mismatch error returned by `Req.Steps.checksum/1`.","ref":"Req.ChecksumMismatchError.html"},{"type":"exception","title":"Req.DecompressError","doc":"Represents an error when decompression fails, returned by `Req.Steps.decompress_body/1`.","ref":"Req.DecompressError.html"},{"type":"exception","title":"Req.HTTPError","doc":"Represents an HTTP protocol error.\n\nThis is a standardised exception that all Req adapters should use for HTTP-protocol-related\nerrors.\n\nThis exception is based on `Mint.HTTPError`.","ref":"Req.HTTPError.html"},{"type":"exception","title":"Req.TooManyRedirectsError","doc":"Represents an error when too many redirects occured, returned by `Req.Steps.redirect/1`.","ref":"Req.TooManyRedirectsError.html"},{"type":"exception","title":"Req.TransportError","doc":"Represents an error with the transport used by an HTTP connection.\n\nThis is a standardised exception that all Req adapters should use for transport-layer-related\nerrors.\n\nThis exception is based on `Mint.TransportError`.","ref":"Req.TransportError.html"},{"type":"extras","title":"Req","doc":"# Req\n\n[![CI](https://github.com/wojtekmach/req/actions/workflows/ci.yml/badge.svg)](https://github.com/wojtekmach/req/actions/workflows/ci.yml)\n[![License](https://img.shields.io/hexpm/l/req.svg)](https://github.com/wojtekmach/req/blob/main/LICENSE.md)\n[![Version](https://img.shields.io/hexpm/v/req.svg)](https://hex.pm/packages/req)\n[![Hex Docs](https://img.shields.io/badge/documentation-gray.svg)](https://hexdocs.pm/req)\n\nReq is a batteries-included HTTP client for Elixir.\n\nWith just a couple lines of code:\n\n```elixir\nMix.install([\n  {:req, \"~> 0.5.0\"}\n])\n\nReq.get!(\"https://api.github.com/repos/wojtekmach/req\").body[\"description\"]\n#=> \"Req is a batteries-included HTTP client for Elixir.\"\n```\n\nwe get automatic response body decompression & decoding, following redirects, retrying on errors,\nand much more. Virtually all of the features are broken down into individual functions called\n_steps_. You can easily re-use and re-arrange built-in steps (see [`Req.Steps`] module) and\nwrite new ones.","ref":"readme.html"},{"type":"extras","title":"Features - Req","doc":"* An easy to use high-level API: [`Req.request/1`], [`Req.new/1`], [`Req.get!/2`], [`Req.post!/2`], etc.\n\n  * Extensibility via request, response, and error steps.\n\n  * Request body compression (via [`compress_body`] step)\n\n  * Automatic response body decompression (via [`compressed`] and [`decompress_body`] steps). Supports gzip, brotli, and zstd.\n\n  * Request body encoding. Supports urlencoded and multipart forms, and JSON. See [`encode_body`].\n\n  * Automatic response body decoding (via [`decode_body`] step.)\n\n  * Encode params as query string (via [`put_params`] step.)\n\n  * Setting base URL (via [`put_base_url`] step.)\n\n  * Templated request paths (via [`put_path_params`] step.)\n\n  * Basic, bearer, and `.netrc` authentication (via [`auth`] step.)\n\n  * Range requests (via [`put_range`]) step.)\n\n  * Use AWS V4 Signature (via [`put_aws_sigv4`]) step.)\n\n  * Request body streaming (by setting `body: enumerable`.)\n\n  * Response body streaming (by setting `into: fun | collectable | :self`.)\n\n  * Follows redirects (via [`redirect`] step.)\n\n  * Retries on errors (via [`retry`] step.)\n\n  * Raise on 4xx/5xx errors (via [`handle_http_errors`] step.)\n\n  * Verify response body against a checksum (via [`checksum`] step.)\n\n  * Basic HTTP caching (via [`cache`] step.)\n\n  * Easily create test stubs (see [`Req.Test`].)\n\n  * Running against a plug (via [`run_plug`] step.)\n\n  * Pluggable adapters. By default, Req uses [Finch] (via [`run_finch`] step.)","ref":"readme.html#features"},{"type":"extras","title":"Usage - Req","doc":"The easiest way to use Req is with [`Mix.install/2`] (requires Elixir v1.12+):\n\n```elixir\nMix.install([\n  {:req, \"~> 0.5.0\"}\n])\n\nReq.get!(\"https://api.github.com/repos/wojtekmach/req\").body[\"description\"]\n#=> \"Req is a batteries-included HTTP client for Elixir.\"\n```\n\nIf you want to use Req in a Mix project, you can add the above dependency to your `mix.exs`.\n\nHere's an example POST with JSON data:\n\n```elixir\niex> Req.post!(\"https://httpbin.org/post\", json: %{x: 1, y: 2}).body[\"json\"]\n%{\"x\" => 1, \"y\" => 2}\n```\n\nYou can stream request body:\n\n```elixir\niex> stream = Stream.duplicate(\"foo\", 3)\niex> Req.post!(\"https://httpbin.org/post\", body: stream).body[\"data\"]\n\"foofoofoo\"\n```\n\nand stream the response body:\n\n```elixir\niex> resp = Req.get!(\"http://httpbin.org/stream/2\", into: IO.stream())\n# output: {\"url\": \"http://httpbin.org/stream/2\", ...}\n# output: {\"url\": \"http://httpbin.org/stream/2\", ...}\niex> resp.status\n200\niex> resp.body\n%IO.Stream{}\n```\n\n(See [`Req`] module documentation for more examples of response body streaming.)\n\nIf you are planning to make several similar requests, you can build up a request struct with\ndesired common options and re-use it:\n\n```elixir\nreq = Req.new(base_url: \"https://api.github.com\")\n\nReq.get!(req, url: \"/repos/sneako/finch\").body[\"description\"]\n#=> \"Elixir HTTP client, focused on performance\"\n\nReq.get!(req, url: \"/repos/elixir-mint/mint\").body[\"description\"]\n#=> \"Functional HTTP client for Elixir with support for HTTP/1 and HTTP/2.\"\n```\n\nSee [`Req.new/1`] for more information on available options.\n\nVirtually all of Req's features are broken down into individual pieces - steps. Req works by running\nthe request struct through these steps. You can easily reuse or rearrange built-in steps or write new\nones. Importantly, steps are just regular functions. Here is another example where we append a request\nstep that inspects the URL just before requesting it:\n\n```elixir\nreq =\n  Req.new(base_url: \"https://api.github.com\")\n  |> Req.Request.append_request_steps(\n    debug_url: fn request ->\n      IO.inspect(URI.to_string(request.url))\n      request\n    end\n  )\n\nReq.get!(req, url: \"/repos/wojtekmach/req\").body[\"description\"]\n# output: \"https://api.github.com/repos/wojtekmach/req\"\n#=> \"Req is a batteries-included HTTP client for Elixir.\"\n```\n\nCustom steps can be packaged into plugins so that they are even easier to use by others. See [Related Packages](#related-packages).\n\nHere is how they can be used:\n\n```elixir\nMix.install([\n  {:req, \"~> 0.5.0\"},\n  {:req_easyhtml, \"~> 0.1.0\"},\n  {:req_s3, \"~> 0.2.3\"},\n  {:req_hex, \"~> 0.2.0\"},\n  {:req_github_oauth, \"~> 0.1.0\"}\n])\n\nreq =\n  (Req.new(http_errors: :raise)\n  |> ReqEasyHTML.attach()\n  |> ReqS3.attach()\n  |> ReqHex.attach()\n  |> ReqGitHubOAuth.attach())\n\nReq.get!(req, url: \"https://elixir-lang.org\").body[\".entry-summary h5\"]\n#=>\n# #EasyHTML[ \n#    Elixir is a dynamic, functional language for building scalable and maintainable applications.\n#   ]\n\nReq.get!(req, url: \"s3://ossci-datasets/mnist/t10k-images-idx3-ubyte.gz\").body\n#=> <<0, 0, 8, 3, ...>>\n\nReq.get!(req, url: \"https://repo.hex.pm/tarballs/req-0.1.0.tar\").body[\"metadata.config\"][\"links\"]\n#=> %{\"GitHub\" => \"https://github.com/wojtekmach/req\"}\n\nReq.get!(req, url: \"https://api.github.com/user\").body[\"login\"]\n# output:\n# paste this user code:\n#\n#   6C44-30A8\n#\n# at:\n#\n#   https://github.com/login/device\n#\n# open browser window? [Yn]\n# 15:22:28.350 [info] response: authorization_pending\n# 15:22:33.519 [info] response: authorization_pending\n# 15:22:38.678 [info] response: authorization_pending\n#=> \"wojtekmach\"\n\nReq.get!(req, url: \"https://api.github.com/user\").body[\"login\"]\n#=> \"wojtekmach\"\n```\n\nSee [`Req.Request`] module documentation for more information on low-level API, request struct, and developing plugins.","ref":"readme.html#usage"},{"type":"extras","title":"Related Packages - Req","doc":"There are many packages that extend the Req library. To get yours listed here, send a PR.\n\n  * [`req_easyhtml`]\n  * [`req_s3`]\n  * [`req_hex`]\n  * [`req_github_oauth`]\n  * [`curl_req`]","ref":"readme.html#related-packages"},{"type":"extras","title":"Presentations - Req","doc":"* [Req: A batteries-included HTTP client for Elixir - ElixirConf 2023, 2023-09-08](https://www.youtube.com/watch?v=owz2QacFuoQ \"ElixirConf 2023 - Wojtek Mach - Req - a batteries-included HTTP client for Elixir\")\n  * [Req: A batteries included HTTP client for Elixir - Elixir Kenya, 2022-08-26](https://www.youtube.com/watch?v=NxWgvHRN6mI \"Req: A batteries included HTTP client for Elixir\")","ref":"readme.html#presentations"},{"type":"extras","title":"Acknowledgments - Req","doc":"Req is built on top of [Finch] and is inspired by [cURL], [Requests], [Tesla], and many other HTTP clients - thank you!","ref":"readme.html#acknowledgments"},{"type":"extras","title":"License - Req","doc":"Copyright (c) 2021 Wojtek Mach\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at [http://www.apache.org/licenses/LICENSE-2.0](http://www.apache.org/licenses/LICENSE-2.0)\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.\n\n[`Req.request/1`]:       https://hexdocs.pm/req/Req.html#request/1\n[`Req.new/1`]:           https://hexdocs.pm/req/Req.html#new/1\n[`Req.get!/2`]:          https://hexdocs.pm/req/Req.html#get!/2\n[`Req.post!/2`]:         https://hexdocs.pm/req/Req.html#post!/2\n[`Req`]:                 https://hexdocs.pm/req\n[`Req.Request`]:         https://hexdocs.pm/req/Req.Request.html\n[`Req.Steps`]:           https://hexdocs.pm/req/Req.Steps.html\n[`Req.Test`]:            https://hexdocs.pm/req/Req.Test.html\n\n[`auth`]:               https://hexdocs.pm/req/Req.Steps.html#auth/1\n[`cache`]:              https://hexdocs.pm/req/Req.Steps.html#cache/1\n[`compress_body`]:      https://hexdocs.pm/req/Req.Steps.html#compress_body/1\n[`compressed`]:         https://hexdocs.pm/req/Req.Steps.html#compressed/1\n[`decode_body`]:        https://hexdocs.pm/req/Req.Steps.html#decode_body/1\n[`decompress_body`]:    https://hexdocs.pm/req/Req.Steps.html#decompress_body/1\n[`encode_body`]:        https://hexdocs.pm/req/Req.Steps.html#encode_body/1\n[`redirect`]:           https://hexdocs.pm/req/Req.Steps.html#redirect/1\n[`handle_http_errors`]: https://hexdocs.pm/req/Req.Steps.html#handle_http_errors/1\n[`output`]:             https://hexdocs.pm/req/Req.Steps.html#output/1\n[`put_base_url`]:       https://hexdocs.pm/req/Req.Steps.html#put_base_url/1\n[`put_params`]:         https://hexdocs.pm/req/Req.Steps.html#put_params/1\n[`put_path_params`]:    https://hexdocs.pm/req/Req.Steps.html#put_path_params/1\n[`run_plug`]:           https://hexdocs.pm/req/Req.Steps.html#run_plug/1\n[`put_range`]:          https://hexdocs.pm/req/Req.Steps.html#put_range/1\n[`put_user_agent`]:     https://hexdocs.pm/req/Req.Steps.html#put_user_agent/1\n[`retry`]:              https://hexdocs.pm/req/Req.Steps.html#retry/1\n[`run_finch`]:          https://hexdocs.pm/req/Req.Steps.html#run_finch/1\n[`checksum`]:           https://hexdocs.pm/req/Req.Steps.html#checksum/1\n[`put_aws_sigv4`]:       https://hexdocs.pm/req/Req.Steps.html#put_aws_sigv4/1\n\n[Finch]: https://github.com/sneako/finch\n[cURL]: https://curl.se\n[Requests]: https://docs.python-requests.org/en/master/\n[Tesla]: https://github.com/elixir-tesla/tesla\n[`req_easyhtml`]: https://github.com/wojtekmach/req_easyhtml\n[`req_s3`]: https://github.com/wojtekmach/req_s3\n[`req_hex`]: https://github.com/wojtekmach/req_hex\n[`req_github_oauth`]: https://github.com/wojtekmach/req_github_oauth\n[`Mix.install/2`]: https://hexdocs.pm/mix/Mix.html#install/2\n[`curl_req`]: https://github.com/derekkraan/curl_req","ref":"readme.html#license"},{"type":"extras","title":"CHANGELOG","doc":"# CHANGELOG","ref":"changelog.html"},{"type":"extras","title":"v0.5.8 (2024-11-29) - CHANGELOG","doc":"* [`Req`]: Check legacy headers when streaming headers\n\n  * [`Req`]: Ignore :into collectable for non-200 responses\n\n  * [`put_aws_sigv4`]: Fix encoding path\n\n  * [`run_finch`]: Add option to configure Finch `:pool_max_idle_time`\n\n  * [`run_finch`]: Prepare for upcoming Finch v0.20","ref":"changelog.html#v0-5-8-2024-11-29"},{"type":"extras","title":"v0.5.7 (2024-10-29) - CHANGELOG","doc":"* [`put_aws_sigv4`]: Fix signature when using custom port\n\n  * [`retry`]: Do not call `retry_delay` fun twice\n\n  * [`auth`]: Support passing a 0-arity function","ref":"changelog.html#v0-5-7-2024-10-29"},{"type":"extras","title":"v0.5.6 (2024-08-01) - CHANGELOG","doc":"* Fix compatibility with Elixir v1.13","ref":"changelog.html#v0-5-6-2024-08-01"},{"type":"extras","title":"v0.5.5 (2024-08-01) - CHANGELOG","doc":"* [`put_aws_sigv4`]: Fix detecting service\n\n  * [`put_aws_sigv4`]: Raise on no `:access_key_id`/`:secret_access_key`/`:service`\n\n  * [`put_aws_sigv4`]: Fix handling `?name` (no value)\n\n  * [`handle_http_errors`]: should run before `verify_checksum`\n\n  * [`encode_body`]: Support `%File.Stream{}` in `:form_multipart`\n\n  * [`encode_body`]: Support `%File.Stream{}` from other nodes in `:form_multipart`","ref":"changelog.html#v0-5-5-2024-08-01"},{"type":"extras","title":"v0.5.4 (2024-07-18) - CHANGELOG","doc":"* [`run_finch`], [`Req.parse_message/2`]: Gracefully handle process messages not meant\n    for the asynchronous response. In that case, `Req.parse_message/2` returns `:unknown`.","ref":"changelog.html#v0-5-4-2024-07-18"},{"type":"extras","title":"v0.5.3 (2024-07-18) - CHANGELOG","doc":"* [`Req.Test`]: Fix using shared mode\n\n  * [`encode_body`]: Add `:form_multipart` option\n\n  * [`put_aws_sigv4`]: Try detecting the service\n\n  * [`run_finch`]: Fix setting `:finch` option","ref":"changelog.html#v0-5-3-2024-07-18"},{"type":"extras","title":"v0.5.2 (2024-07-08) - CHANGELOG","doc":"* [`put_aws_sigv4`]: Fix bug when using custom headers\n\n  * [`put_aws_sigv4`]: Add `:token` option\n\n  * [`redirect`]: Cancel async request before redirecting\n\n  * [`decode_body`]: Support `application/zstd` and `.zst`","ref":"changelog.html#v0-5-2-2024-07-08"},{"type":"extras","title":"v0.5.1 (2024-06-24) - CHANGELOG","doc":"* [`retry`]: Default `:retry_log_level` to `:warning`\n\n  * [`put_path_params`]: Add `:path_params_style` option\n\n  * [`put_aws_sigv4`]: Fix path encoding\n\n  * [`decode_body`]: Improve tar detection\n\n  * [`run_finch`]: Fix defaulting to using just HTTP/1","ref":"changelog.html#v0-5-1-2024-06-24"},{"type":"extras","title":"v0.5.0 (2024-05-28) - CHANGELOG","doc":"Req v0.5.0 brings testing enhancements, errors standardization, `%Req.Response.Async{}`, and more improvements and bug fixes.","ref":"changelog.html#v0-5-0-2024-05-28"},{"type":"extras","title":"Testing Enhancements - CHANGELOG","doc":"In previous releases, we could only create test _stubs_ (using [`Req.Test.stub/2`]), that is, fake\nHTTP servers which had predefined behaviour. Let's say we're integrating with a third-party\nweather service and we might create a stub for it like below:\n\n```elixir\nReq.Test.stub(MyApp.Weather, fn conn ->\n  Req.Test.json(conn, %{\"celsius\" => 25.0})\nend)\n```\n\nAnytime we hit this fake we'll get the same result. This works extremely well for simple\nintegrations however it's not quite enough for more complicated ones. Imagine we're using\nsomething like AWS S3 and we test uploading some data and reading it back again. While we could do\nthis:\n\n```elixir\nReq.Test.stub(MyApp.S3, fn\n  conn when conn.method == \"PUT\" ->\n    # ...\n\n  conn when conn.method == \"GET\" ->\n    # ...\nend)\n```\n\nmaking the test just a little bit more thorough will make it MUCH more complicated, for example:\nthe first GET request should return a 404, we then make a PUT, and now GET should return a 200.\nWe could solve it by adding some state to our test (e.g. an agent) but there is a simpler way and\nthat is to set request expectations using the new [`Req.Test.expect/3`] function:\n\n```elixir\nReq.Test.expect(MyApp.S3, fn conn when conn.method == \"GET\" ->\n  Plug.Conn.send_resp(conn, 404, \"not found\")\nend)\n\nReq.Test.expect(MyApp.S3, fn conn when conn.method == \"PUT\" ->\n  {:ok, body, conn} = Plug.Conn.read_body(conn)\n  assert body == \"foo\"\n  Plug.Conn.send_resp(conn, 200, \"\")\nend)\n\nReq.Test.expect(MyApp.S3, fn conn when conn.method == \"GET\" ->\n  Plug.Conn.send_resp(conn, 200, \"foo\")\nend)\n```\n\nThe important part is the request expectations are meant to run in order (and fail if they don't).\n\nIn this release we're also adding [`Req.Test.transport_error/2`], a way to simulate network\nerrors.\n\nHere is another example using both of the new features, let's simulate a server that is\nhaving issues: on the first request it is not responding and on the following two requests it\nreturns an HTTP 500. Only on the fourth request it returns an HTTP 200. Req by default\nautomatically retries transient errors (using [`retry`] step) so it will make multiple\nrequests exercising all of our request expectations:\n\n```elixir\niex> Req.Test.expect(MyApp.S3, &Req.Test.transport_error(&1, :econnrefused))\niex> Req.Test.expect(MyApp.S3, 2, &Plug.Conn.send_resp(&1, 500, \"internal server error\"))\niex> Req.Test.expect(MyApp.S3, &Plug.Conn.send_resp(&1, 200, \"ok\"))\niex> Req.get!(plug: {Req.Test, MyApp.S3}).body\n# 15:57:06.309 [error] retry: got exception, will retry in 1000ms, 3 attempts left\n# 15:57:06.309 [error] ** (Req.TransportError) connection refused\n# 15:57:07.310 [error] retry: got response with status 500, will retry in 2000ms, 2 attempts left\n# 15:57:09.311 [error] retry: got response with status 500, will retry in 4000ms, 1 attempt left\n\"ok\"\n```\n\nFinally, for parity with [Mox](https://hexdocs.pm/mox), we add functions for setting ownership\nmode:\n\n  * [`Req.Test.set_req_test_from_context/1`]\n  * [`Req.Test.set_req_test_to_private/1`]\n  * [`Req.Test.set_req_test_to_shared/1`]\n\nAnd for verifying expectations:\n\n  * [`Req.Test.verify!/0`]\n  * [`Req.Test.verify!/1`]\n  * [`Req.Test.verify_on_exit!/1`]\n\nThanks to Andrea Leopardi for driving the testing improvements.","ref":"changelog.html#testing-enhancements"},{"type":"extras","title":"Standardized Errors - CHANGELOG","doc":"In previous releases, when using the default adapter, Finch, Req could return these exceptions on\nnetwork/protocol errors: `Mint.TransportError`, `Mint.HTTPError`, and `Finch.Error`. They have\nnow been standardized into: [`Req.TransportError`] and [`Req.HTTPError`] for more consistent\nexperience. In fact, this standardization was the pre-requisite of adding\n[`Req.Test.transport_error/2`]!\n\nTwo additional exception structs have been added: [`Req.ArchiveError`] and [`Req.DecompressError`]\nfor zip/tar/etc errors in [`decode_body`] and gzip/br/zstd/etc errors in [`decompress_body`]\nrespectively. Additionally, [`decode_body`] now returns `Jason.DecodeError` instead of raising it.\n\n### `%Req.Response.Async{}`\n\nIn previous releases we added ability to stream response body chunks into the current process\nmailbox using the `into: :self` option. When such is used, the `response.body` is now set to\n[`Req.Response.Async`] struct which implements the [`Enumerable`] protocol.\n\nHere's a quick example:\n\n```elixir\nresp = Req.get!(\"http://httpbin.org/stream/2\", into: :self)\nresp.body\n#=> #Req.Response.Async<...>\nEnum.each(resp.body, &IO.puts/1)\n# {\"url\": \"http://httpbin.org/stream/2\", ..., \"id\": 0}\n# {\"url\": \"http://httpbin.org/stream/2\", ..., \"id\": 1}\n```\n\nHere is another example where we use Req to talk to two different servers. The first server\nproduces some test data, strings `\"foo\"`, `\"bar\"` and `\"baz\"`. The second one is an \"echo\" server, it simply\nresponds with the request body it returned. We then stream data from one server, transform it, and\nstream it to the other one:\n\n```elixir\nMix.install([\n  {:req, \"~> 0.5\"},\n  {:bandit, \"~> 1.0\"}\n])\n\n{:ok, _} =\n  Bandit.start_link(\n    scheme: :http,\n    port: 4000,\n    plug: fn conn, _ ->\n      conn = Plug.Conn.send_chunked(conn, 200)\n      {:ok, conn} = Plug.Conn.chunk(conn, \"foo\")\n      {:ok, conn} = Plug.Conn.chunk(conn, \"bar\")\n      {:ok, conn} = Plug.Conn.chunk(conn, \"baz\")\n      conn\n    end\n  )\n\n{:ok, _} =\n  Bandit.start_link(\n    scheme: :http,\n    port: 4001,\n    plug: fn conn, _ ->\n      {:ok, body, conn} = Plug.Conn.read_body(conn)\n      Plug.Conn.send_resp(conn, 200, body)\n    end\n  )\n\nresp = Req.get!(\"http://localhost:4000\", into: :self)\nstream = resp.body |> Stream.with_index() |> Stream.map(fn {data, idx} -> \"[#{idx}]#{data}\" end)\nReq.put!(\"http://localhost:4001\", body: stream).body\n#=> \"[0]foo[1]bar[2]baz\"\n```\n\n`Req.Response.Async` is an experimental feature which may change in the future.\n\nThe existing caveats to `into: :self` still apply, that is:\n\n  * If the request is sent using HTTP/1, an extra process is spawned to consume messages from the\n    underlying socket.\n\n  * On both HTTP/1 and HTTP/2 the messages are sent to the current process as soon as they arrive,\n    as a firehose with no back-pressure.\n\nIf you wish to maximize request rate or have more control over how messages are streamed, use\n`into: fun` or `into: collectable` instead.","ref":"changelog.html#standardized-errors"},{"type":"extras","title":"Full v0.5.0 CHANGELOG - CHANGELOG","doc":"* [`Req`]: Deprecate setting `:headers` to values other than string/integer/`DateTime`.\n    This is to potentially allow special handling of atom values in the future.\n\n  * [`Req`]: Add `Req.run/2` and `Req.run!/2`.\n\n  * [`Req`]: `into: :self` now sets `response.body` as `Req.Response.Async` which implements\n    enumerable.\n\n  * [`Req.Request`]: Deprecate setting `:redact_auth`. It now has no effect. Instead of allowing\n    to opt out of, we give an idea what the secret was without revealing it fully:\n\n    ```elixir\n    iex> Req.new(auth: {:basic, \"foobar:baz\"})\n    %Req.Request{\n      options: %{auth: {:basic, \"foo*******\"}},\n      ...\n    }\n\n    iex> Req.new(headers: [authorization: \"bearer foobarbaz\"])\n    %Req.Request{\n      headers: %{\"authorization\" => [\"bearer foo******\"]},\n      ...\n    }\n    ```\n\n  * [`Req.Request`]: Deprecate `halt/1` in favour of `Req.Request.halt/2`.\n\n  * [`Req.Test`]: Add [`Req.Test.transport_error/2`] to simulate transport errors.\n\n  * [`Req.Test`]: Add [`Req.Test.expect/3`].\n\n  * [`Req.Test`]: Add functions for setting ownership mode: [`Req.Test.set_req_test_from_context/1`], [`Req.Test.set_req_test_to_private/1`],\n    [`Req.Test.set_req_test_to_shared/1`] and for verifying expectations: [`Req.Test.verify!/0`], [`Req.Test.verify!/1`], and [`Req.Test.verify_on_exit!/1`].\n\n  * [`Req.Test`]: Add [`Req.Test.html/2`].\n\n  * [`Req.Test`]: Add [`Req.Test.text/2`].\n\n  * [`Req.Test`]: Drop `:nimble_ownership` dependency.\n\n  * [`Req.Test`]: Deprecate `Req.Test.stub/1`, i.e. the intended use case is to only work\n    with _plug_ stubs/mocks.\n\n  * [`decode_body`]: Return `Jason.DecodeError` on JSON errors instead of raising it.\n\n  * [`decode_body`]: Return [`Req.ArchiveError`] on tar/zip errors.\n\n  * [`decompress_body`]: Return [`Req.DecompressError`].\n\n  * [`put_aws_sigv4`]: Drop `:aws_signature` dependency.\n\n  * [`retry`]: **(BREAKING CHANGE)** Consider\n    `%Req.TransportError{reason: :closed | :econnrefused | :timeout}` as transient. Previously\n    _any_ exceptions with those reason values were consider as such.\n\n  * [`retry`]: **(BREAKING CHANGE)** Consider\n    `%Req.HTTPError{protocol: :http2, reason: :unprocessed}` as transient.\n\n  * [`run_finch`]: **(BREAKING CHANGE)** Return [`Req.HTTPError`] instead of `Mint.HTTPError`.\n\n  * [`run_finch`]: **(BREAKING CHANGE)** Return [`Req.TransportError`] instead of `Mint.TransportError`.\n\n  * [`run_finch`]: Set `inet6: true` if URL looks like IPv6 address.\n\n  * [`put_plug`]: Move most documentation to [`run_plug`].\n\n  * [`run_plug`]: Make public.\n\n  * [`run_plug`]: Add support for simulating network issues using [`Req.Test.transport_error/2`].\n\n  * [`run_plug`]: Support passing 2-arity functions as plugs.\n\n  * [`run_plug`]: Automatically fetch query params.\n\n  * [`verify_checksum`]: Fix handling compressed responses.","ref":"changelog.html#full-v0-5-0-changelog"},{"type":"extras","title":"v0.4.14 (2024-03-15) - CHANGELOG","doc":"* [`redirect`]: Return [`Req.TooManyRedirectsError`] exception.\n\n    Previously we _always_ raised a `RuntimeError`. Besides changing the exception struct, now\n    it is _returned_:\n\n        iex> Req.get(\"https://httpbin.org/redirect/4\", max_redirects: 3)\n        # 07:08:06.868 [debug] redirecting to /relative-redirect/3\n        # 07:08:06.988 [debug] redirecting to /relative-redirect/2\n        # 07:08:07.109 [debug] redirecting to /relative-redirect/1\n        {:error, %Req.TooManyRedirectsError{max_redirects: 3}}\n\n    When users where using functions like `Req.get!`, the exception will of course still be\n    raised.\n\n  * Relax `nimble_ownership` version requirement\n\n  * [`Req.Test`]: Allow plug stub to be a `module` or `{module, options}`\n\n  * [`Req.Test`]: Document stubbing with Broadway","ref":"changelog.html#v0-4-14-2024-03-15"},{"type":"extras","title":"v0.4.13 (2024-03-07) - CHANGELOG","doc":"* [`run_finch`]: Default to `connect_options: [protocols: [:http1]]` due to regression\n    with HTTP/2 requests over HTTP/1 connections (`protocols: [:http1, :http2]`) with request body\n    size exceeding 64kib.","ref":"changelog.html#v0-4-13-2024-03-07"},{"type":"extras","title":"v0.4.12 (2024-03-06) - CHANGELOG","doc":"* [`Req`]: Add response body streaming via `into: :self`, [`Req.parse_message/2`],\n    and `Req.cancel_async_response/1`.\n\n  * [`Req`]: Deprecate `Req.update/2` in favour of [`Req.merge/2`]\n\n  * [`Req.Test`]: Add [`Req.Test.allow/3`]\n\n  * [`compressed`]: Default `compressed: false` when streaming response body\n\n  * [`put_base_url`]: Allow `:base_url` to be a 0-arity function or MFArgs\n\n  * [`put_plug`]: Do not leak `Plug.Test` messages","ref":"changelog.html#v0-4-12-2024-03-06"},{"type":"extras","title":"v0.4.11 (2024-02-19) - CHANGELOG","doc":"* [`Req.Test.json/2`]: Don't crash compilation when Plug is not available","ref":"changelog.html#v0-4-11-2024-02-19"},{"type":"extras","title":"v0.4.10 (2024-02-19) - CHANGELOG","doc":"* [`run_finch`]: Default to `connect_options: [protocols: [:http1, :http2]]`.\n\n  * [`run_finch`]: Change version requirement to `~> 0.17`, that is all versions up to `1.0`.\n\n  * [`put_aws_sigv4`]: Support streaming request body.\n\n  * [`auth`]: Always update `authorization` header.\n\n  * [`decode_body`]: Gracefully handle multiple content-type values.\n\n  * [`Req.Request.new/1`]: Use `URI.parse` for now.","ref":"changelog.html#v0-4-10-2024-02-19"},{"type":"extras","title":"v0.4.9 (2024-02-14) - CHANGELOG","doc":"* [`retry`]: Raise on invalid return from `:retry_delay` function\n\n * [`run_finch`]: Update to Finch 0.17\n\n * [`run_finch`]: Deprecate `connect_options: [protocol: ...]` in favour of\n   `connect_options: [protocols: ...]]` which defaults to `[:http1, :http2]`, that is,\n   make request using HTTP/1 but if negotiated switch to HTTP/2 over the HTTP/1 connection.\n\n * New step: [`put_aws_sigv4`] - signs request with AWS Signature Version 4.","ref":"changelog.html#v0-4-9-2024-02-14"},{"type":"extras","title":"v0.4.8 (2023-12-11) - CHANGELOG","doc":"* [`put_plug`]: Fix response streaming. Previously we were relying on unreleased\n    Plug features (which may never get released). Now, Plug adapter will emit the\n    entire response body as one chunk. Thus,\n    `plug: plug, into: fn ... -> {:halt, acc} end` is not yet supported as it\n    requires Plug changes that are still being discussed. On the flip side,\n    we should have much more stable Plug integration regardless of this small\n    limitation.","ref":"changelog.html#v0-4-8-2023-12-11"},{"type":"extras","title":"v0.4.7 (2023-12-11) - CHANGELOG","doc":"* [`put_plug`]: Don't crash if plug is not installed and :plug is not used","ref":"changelog.html#v0-4-7-2023-12-11"},{"type":"extras","title":"v0.4.6 (2023-12-11) - CHANGELOG","doc":"* New step: [`checksum`]\n  * [`put_plug`]: Fix response streaming when plug uses `send_resp` or `send_file`\n  * [`retry`]: Retry on `:closed`","ref":"changelog.html#v0-4-6-2023-12-11"},{"type":"extras","title":"v0.4.5 (2023-10-27) - CHANGELOG","doc":"* `decompress_body`: Remove `content-length` header\n\n  * `auth`: Deprecate `auth: {user, pass}` in favour of `auth: {:basic, \"user:pass\"}`\n\n  * `Req.Request`: Allow steps to be `{mod, fun, args}`","ref":"changelog.html#v0-4-5-2023-10-27"},{"type":"extras","title":"v0.4.4 (2023-10-05) - CHANGELOG","doc":"* `compressed`: Check for optional depenedencies brotli and ezstd only at compile-time.\n    (backported from v0.3.12.)\n\n  * `decode_body`: Check for optional depenedency nimble_csv at compile-time.\n    (backported from v0.3.12.)\n\n  * `run_finch`: Add `:finch_private` option","ref":"changelog.html#v0-4-4-2023-10-05"},{"type":"extras","title":"v0.4.3 (2023-09-13) - CHANGELOG","doc":"* [`Req.new/1`]: Fix setting `:redact_auth`\n\n  * [`Req.Request`]: Add `Req.Request.get_option_lazy/3`\n\n  * [`Req.Request`]: Add `Req.Request.drop_options/2`","ref":"changelog.html#v0-4-3-2023-09-13"},{"type":"extras","title":"v0.4.2 (2023-09-04) - CHANGELOG","doc":"* [`put_plug`]: Handle response streaming on Plug 1.15+.\n\n  * Don't warn on mixed-case header names","ref":"changelog.html#v0-4-2-2023-09-04"},{"type":"extras","title":"v0.4.1 (2023-09-01) - CHANGELOG","doc":"* Fix Req.Request Inspect regression","ref":"changelog.html#v0-4-1-2023-09-01"},{"type":"extras","title":"v0.4.0 (2023-09-01) - CHANGELOG","doc":"Req v0.4.0 changes headers to be maps, adds request & response streaming, and improves steps.","ref":"changelog.html#v0-4-0-2023-09-01"},{"type":"extras","title":"Change Headers to be Maps - CHANGELOG","doc":"Previously headers were lists of name/value tuples, e.g.:\n\n```elixir\n[{\"content-type\", \"text/html\"}]\n```\n\nThis is a standard across the ecosystem (with minor difference that some Erlang libraries use\ncharlists instead of binaries.)\n\nThere are some problems with this particular choice though:\n\n  * We cannot use `headers[name]`\n  * We cannot use pattern matching\n\nIn short, this representation isn't very ergonomic to use.\n\nNow headers are maps of string names and lists of values, e.g.:\n\n```elixir\n%{\"content-type\" => [\"text/html\"]}\n```\n\nThis allows `headers[name]` usage:\n\n```elixir\nresponse.headers[\"content-type\"]\n#=> [\"text/html\"]\n```\n\nand pattern matching:\n\n```elixir\ncase Req.request!(req) do\n  %{headers: %{\"content-type\" => [\"application/json\" <> _]}} ->\n    # handle JSON response\nend\n```\n\nThis is a major breaking change. If you cannot easily update your app\nor your dependencies, do:\n\n```elixir\n# config/config.exs\nconfig :req, legacy_headers_as_lists: true\n```\n\nThis legacy fallback will be removed on Req 1.0.\n\nThere are two other changes to headers in this release.\n\nHeader names are now case-insensitive in functions like\n`Req.Response.get_header/2`.\n\nTrailer headers, or more precisely trailer fields or simply trailers, are now stored\nin a separate `trailers` field on the `%Req.Response{}` struct as long as you use Finch 0.17+.","ref":"changelog.html#change-headers-to-be-maps"},{"type":"extras","title":"Add Request Body Streaming - CHANGELOG","doc":"Req v0.4 adds official support for request body streaming by setting the request body to an\n`enumerable`. Here's an example:\n\n```elixir\niex> stream = Stream.duplicate(\"foo\", 3)\niex> Req.post!(\"https://httpbin.org/post\", body: stream).body[\"data\"]\n\"foofoofoo\"\n```\n\nThe enumerable is passed through request steps and they may change it. For example,\nthe [`compress_body`] step gzips the request body on the fly.","ref":"changelog.html#add-request-body-streaming"},{"type":"extras","title":"Add Response Body Streaming - CHANGELOG","doc":"Req v0.4 also adds response body streaming, via the `:into` option.\n\nHere's an example where we download the first 20kb (by making a _range_ request, via the\n[`put_range`] step) of Elixir release zip. We stream the response body into a function\nand can handle each body chunk. The function receives a `{:data, data}, {req, resp}` and returns\na `{:cont | :halt, {req, resp}}` tuple.\n\n```elixir\nresp =\n  Req.get!(\n    url: \"https://github.com/elixir-lang/elixir/releases/download/v1.15.4/elixir-otp-26.zip\",\n    range: 0..20_000,\n    into: fn {:data, data}, {req, resp} ->\n      IO.inspect(byte_size(data), label: :chunk)\n      {:cont, {req, resp}}\n    end\n  )\n\n# output: 17:07:38.131 [debug] redirecting to https://objects.githubusercontent.com/github-production-release-asset-2e6(...)\n# output: chunk: 16384\n# output: chunk: 3617\n\nresp.status #=> 206\nresp.headers[\"content-range\"] #=> [\"bytes 0-20000/6801977\"]\nresp.body #=> \"\"\n```\n\nNotice we only stream response _body_, that is, Req automatically handles HTTP response status and\nheaders. Once the stream is done, Req passes the response through response steps which allows\nfollowing redirects, retrying on errors, etc. Response `body` is set to empty string `\"\"`\nwhich is then ignored by [`decompress_body`], [`decode_body`], and similar steps. If you need\nto decompress or decode incoming chunks, you need to do that in your custom `into: fun` function.\n\nAs the name `:into` implies, we can also stream response body into any [`Collectable`].\nHere's a similar snippet to above where we stream to a file:\n\n```elixir\nresp =\n  Req.get!(\n    url: \"https://github.com/elixir-lang/elixir/releases/download/v1.15.4/elixir-otp-26.zip\",\n    range: 0..20_000,\n    into: File.stream!(\"elixit-otp-26.zip.1\")\n  )\n\n# output: 17:07:38.131 [debug] redirecting to (...)\nresp.status #=> 206\nresp.headers[\"content-range\"] #=> [\"bytes 0-20000/6801977\"]\nresp.body #=> %File.Stream{}\n```","ref":"changelog.html#add-response-body-streaming"},{"type":"extras","title":"Full CHANGELOG - CHANGELOG","doc":"* Change `request.headers` and `response.headers` to be maps.\n\n  * Ensure `request.headers` and `response.headers` are downcased.\n\n    Per [RFC 9110: HTTP Semantics](https://www.rfc-editor.org/rfc/rfc9110.html),\n    HTTP headers should be case-insensitive. However, per\n    [RFC 9113: HTTP/2](https://datatracker.ietf.org/doc/html/rfc9113) headers\n    must be sent downcased.\n\n    Req headers are now stored internally downcased and all accessor functions\n    like [`Req.Response.get_header/2`] are downcasing the given header name.\n\n  * Add `trailers` field to [`Req.Response`] struct. Trailer field is only filled in on Finch 0.17+.\n\n  * Make `request.registered_options` internal representation private.\n\n  * Make `request.options` internal representation private.\n\n    Currently `request.options` field is a map but it may change in the future.\n    One possible future change is using keywords lists internally which would\n    allow, for example, `Req.new(params: [a: 1]) |> Req.merge(params: [b: 2])`\n    to keep duplicate `:params` in `request.options` which would then allow to\n    decide the duplicate key semantics on a per-step basis. And so, for example,\n    [`put_params`] would _merge_ params but most steps would simply use the\n    first value.\n\n    To have some room for manoeuvre in the future we should stop pattern\n    matching on `request.options`. Calling `request.options[key]`,\n    `put_in(request.options[key], value)`, and\n    `update_in(request.options[key], fun)` _is_ allowed.\n\n  * Fix typespecs for some functions\n\n  * Deprecate `output` step in favour of `into: File.stream!(path)`.\n\n  * Rename `follow_redirects` step to [`redirect`]\n\n  * [`redirect`]: Rename `:follow_redirects` option to `:redirect`.\n\n  * [`redirect`]: Rename `:location_trusted` option to `:redirect_trusted`.\n\n  * [`redirect`]: Change HTTP request method to GET only on POST requests that result in 301..303.\n\n    Previously we were changing the method to GET for all 3xx except 307 and 308.\n\n  * [`decompress_body`]: Remove support for `deflate` compression (which was broken)\n\n  * [`decompress_body`]: Don't crash on unknown codec\n\n  * [`decompress_body`]: Fix handling HEAD requests\n\n  * [`decompress_body`]: Re-calculate `content-length` header after decompresion\n\n  * [`decompress_body`]: Remove `content-encoding` header after decompression\n\n  * [`decode_body`]: Do not decode response with `content-encoding` header\n\n  * [`run_finch`]: Add `:inet6` option\n\n  * [`retry`]: Support `retry: :safe_transient` which retries HTTP 408/429/500/502/503/504\n    or exceptions with `reason` field set to `:timeout`/`:econnrefused`.\n\n    `:safe_transient` is the new default retry mode. (Previously we retried on 408/429/5xx and\n    _any_ exception.)\n\n  * [`retry`]: Support `retry: :transient` which is the same as `:safe_transient` except\n    it retries on all HTTP methods\n\n  * [`retry`]: Use `retry-after` header value on HTTP 503 Service Unavailable. Previously\n    only HTTP 429 Too Many Requests was using this header value.\n\n  * [`retry`]: Support `retry: &fun/2`. The function receives `request, response_or_exception`\n    and returns either:\n\n      * `true` - retry with the default delay\n\n      * `{:delay, milliseconds}` - retry with the given delay\n\n      * `false/nil` - don't retry\n\n  * [`retry`]: Deprecate `retry: :safe` in favour of `retry: :safe_transient`\n\n  * [`retry`]: Deprecate `retry: :never` in favour of `retry: false`\n\n  * [`Req.request/2`]: Improve error message on invalid arguments\n\n  * [`Req.merge/2`]: Do not duplicate headers\n\n  * [`Req.merge/2`]: Merge `:params`\n\n  * [`Req.Request`]: Fix displaying redacted basic authentication\n\n  * [`Req.Request`]: Add [`Req.Request.get_option/3`]\n\n  * [`Req.Request`]: Add [`Req.Request.fetch_option/2`]\n\n  * [`Req.Request`]: Add [`Req.Request.fetch_option!/2`]\n\n  * [`Req.Request`]: Add [`Req.Request.delete_option/2`]\n\n  * [`Req.Response`]: Add [`Req.Response.delete_header/2`]\n\n  * [`Req.Response`]: Add [`Req.Response.update_private/4`]","ref":"changelog.html#full-changelog"},{"type":"extras","title":"v0.3.12 (2023-08-05) - CHANGELOG","doc":"* `compressed`: Check for optional depenedencies brotli and ezstd only at compile-time.\n  * `decode_body`: Check for optional depenedency nimble_csv at compile-time.","ref":"changelog.html#v0-3-12-2023-08-05"},{"type":"extras","title":"v0.3.11 (2023-07-24) - CHANGELOG","doc":"* Support `Req.get(options)`, `Req.post(options)`, etc\n  * Add [`Req.Request.new/1`]\n  * [`retry`]: Fix returning correct `private.req_retry_count`","ref":"changelog.html#v0-3-11-2023-07-24"},{"type":"extras","title":"v0.3.10 (2023-06-20) - CHANGELOG","doc":"* [`decompress_body`]: No-op on non-binary response body\n  * [`decompress_body`]: Support multiple `content-encoding` headers\n  * [`decode_body`]: Remove `:extract` option\n  * Remove deprecated `Req.post!(url, body)` and similar functions","ref":"changelog.html#v0-3-10-2023-06-20"},{"type":"extras","title":"v0.3.9 (2023-06-08) - CHANGELOG","doc":"* [`put_path_params`]: URI-encode path params","ref":"changelog.html#v0-3-9-2023-06-08"},{"type":"extras","title":"v0.3.8 (2023-05-22) - CHANGELOG","doc":"* Add `:redact_auth` option to redact auth credentials, defaults to `true`.\n  * Soft-deprecate `Req.Request.run,run!` in favour of [`Req.Request.run_request/1`].","ref":"changelog.html#v0-3-8-2023-05-22"},{"type":"extras","title":"v0.3.7 (2023-05-18) - CHANGELOG","doc":"* Deprecate setting headers to `%NaiveDateTime{}`, always use `%DateTime{}`.\n  * [`decode_body`]: Add `:decode_json` option\n  * [`follow_redirects`]: Add `:redirect_log_level`\n  * [`follow_redirects`]: Preserve HTTP method on 307/308 redirects\n  * [`run_finch`]: Allow `:finch_request` to perform the underlying request. This deprecates\n    passing 1-arity function `f(finch_request)` in favour of 4-arity\n    `f(request, finch_request, finch_name, finch_options)`.","ref":"changelog.html#v0-3-7-2023-05-18"},{"type":"extras","title":"v0.3.6 (2023-03-06) - CHANGELOG","doc":"* [`run_finch`]: Fix setting `:hostname` option\n  * [`decode_body`]: Add `:extract` option to automatically extract archives (zip, tar, etc)","ref":"changelog.html#v0-3-6-2023-03-06"},{"type":"extras","title":"v0.3.5 (2023-02-01) - CHANGELOG","doc":"* New step: [`put_path_params`]\n  * [`auth`]: Accept string","ref":"changelog.html#v0-3-5-2023-02-01"},{"type":"extras","title":"v0.3.4 (2023-01-03) - CHANGELOG","doc":"* [`retry`]: Add `:retry_log_level` option","ref":"changelog.html#v0-3-4-2023-01-03"},{"type":"extras","title":"v0.3.3 (2022-12-08) - CHANGELOG","doc":"* [`follow_redirects`]: Inherit scheme from previous location\n  * [`run_finch`]: Fix setting connect timeout\n  * [`run_finch`]: Add `:finch_request` option","ref":"changelog.html#v0-3-3-2022-12-08"},{"type":"extras","title":"v0.3.2 (2022-11-14) - CHANGELOG","doc":"* [`decode_body`]: Decode JSON when response is json-api mime type\n  * [`put_params`]: Fix bug when params have been duplicated when retrying requeset\n  * [`retry`]: Remove `retry: :always` option\n  * [`retry`]: Soft-deprecate `retry: :never` in favour of `retry: false`\n  * [`run_finch`]: Add `:transport_opts`, `:proxy_headers`, `:proxy`, and `:client_settings` options\n  * `Req.Response.json/2`: Do not override content-type","ref":"changelog.html#v0-3-2-2022-11-14"},{"type":"extras","title":"v0.3.1 (2022-09-09) - CHANGELOG","doc":"* [`encode_body`]: Set Accept header in JSON requests\n  * [`put_base_url`]: Fix merging with leading and/or trailing slashes\n  * Fix merging :adapter option\n  * Add get/2, post/2, put/2, patch/2, delete/2 and head/2","ref":"changelog.html#v0-3-1-2022-09-09"},{"type":"extras","title":"v0.3.0 (2022-06-21) - CHANGELOG","doc":"Req v0.3.0 brings redesigned API, new steps, and improvements to existing steps.","ref":"changelog.html#v0-3-0-2022-06-21"},{"type":"extras","title":"New API - CHANGELOG","doc":"The new API allows building a request struct with all the built-in steps. It can be then piped\nto functions like `Req.get!/2`:\n\n```elixir\niex> req = Req.new(base_url: \"https://api.github.com\")\n\niex> req |> Req.get!(url: \"/repos/sneako/finch\") |> then(& &1.body[\"description\"])\n\"Elixir HTTP client, focused on performance\"\n\niex> req |> Req.get(url: \"/repos/elixir-mint/mint\") |> then(& &1.body[\"description\"])\n\"Functional HTTP client for Elixir with support for HTTP/1 and HTTP/2.\"\n```\n\nSetting body and encoding it to form/JSON is now done through `:body/:form/:json` options:\n\n```elixir\niex> Req.post!(\"https://httpbin.org/anything\", body: \"hello!\").body[\"data\"]\n\"hello!\"\n\niex> req = Req.new(url: \"https://httpbin.org/anything\")\niex> Req.post!(req, form: [x: 1]).body[\"form\"]\n%{\"x\" => \"1\"}\niex> Req.post!(req, json: %{x: 2}).body[\"form\"]\n%{\"x\" => 2}\n```","ref":"changelog.html#new-api"},{"type":"extras","title":"Improved Error Handling - CHANGELOG","doc":"Req now validates option names ensuring users didn't accidentally mistyped them.\nIf they did, it will try to give a helpful error message. Here are some examples:\n\n```elixir\nReq.request!(urll: \"https://httpbin.org\")\n** (ArgumentError) unknown option :urll. Did you mean :url?\n\nReq.new(bas_url: \"https://httpbin.org\")\n** (ArgumentError) unknown option :bas_url. Did you mean :base_url?\n```\n\nReq also has a new option to handle HTTP errors (4xx/5xx). By default it will continue to\nreturn the error responses:\n\n```elixir\nReq.get!(\"https://httpbin.org/status/404\")\n#=> %Req.Response{status: 404, ...}\n```\n\nbut users can now pass `http_errors: :raise` to raise an exception instead:\n\n```elixir\nReq.get!(\"https://httpbin.org/status/404\", http_errors: :raise)\n** (RuntimeError) The requested URL returned error: 404\nResponse body: \"\"\n```\n\nThis is especially useful in one-off scripts where we only really care about the\n\"happy path\" but would still like to get a good error message when something\nunexpected happened.","ref":"changelog.html#improved-error-handling"},{"type":"extras","title":"Plugins - CHANGELOG","doc":"From the very beginning, Req could be extended with custom steps. To make using such custom steps\nby others even easier, they can be packaged up into plugins.\n\nHere are some examples:\n\n  * [`req_easyhtml`](https://github.com/wojtekmach/req_easyhtml)\n  * [`req_s3`](https://github.com/wojtekmach/req_s3)\n  * [`req_hex`](https://github.com/wojtekmach/req_hex)\n  * [`req_github_oauth`](https://github.com/wojtekmach/req_github_oauth)\n\nAnd here's how they can be used:\n\n```elixir\nMix.install([\n  {:req, \"~> 0.3.0\"},\n  {:req_easyhtml, github: \"wojtekmach/req_easyhtml\"},\n  {:req_s3, github: \"wojtekmach/req_s3\"},\n  {:req_hex, github: \"wojtekmach/req_hex\"},\n  {:req_github_oauth, github: \"wojtekmach/req_github_oauth\"}\n])\n\nreq =\n  (Req.new(http_errors: :raise)\n  |> ReqEasyHTML.attach()\n  |> ReqS3.attach()\n  |> ReqHex.attach()\n  |> ReqGitHubOAuth.attach())\n\nReq.get!(req, url: \"https://elixir-lang.org\").body[\".entry-summary h5\"]\n#=>\n# #EasyHTML[ \n#    Elixir is a dynamic, functional language for building scalable and maintainable applications.\n#   ]\n\nReq.get!(req, url: \"s3://ossci-datasets\").body\n#=>\n# [\n#   \"mnist/\",\n#   \"mnist/t10k-images-idx3-ubyte.gz\",\n#   \"mnist/t10k-labels-idx1-ubyte.gz\",\n#   \"mnist/train-images-idx3-ubyte.gz\",\n#   \"mnist/train-labels-idx1-ubyte.gz\"\n# ]\n\nReq.get!(req, url: \"https://repo.hex.pm/tarballs/req-0.1.0.tar\").body[\"metadata.config\"][\"links\"]\n#=> %{\"GitHub\" => \"https://github.com/wojtekmach/req\"}\n\nReq.get!(req, url: \"https://api.github.com/user\").body[\"login\"]\n# Outputs:\n# paste this user code:\n#\n#   6C44-30A8\n#\n# at:\n#\n#   https://github.com/login/device\n#\n# open browser window? [Yn]\n# 15:22:28.350 [info] response: authorization_pending\n# 15:22:33.519 [info] response: authorization_pending\n# 15:22:38.678 [info] response: authorization_pending\n#=> \"wojtekmach\"\n\nReq.get!(req, url: \"https://api.github.com/user\").body[\"login\"]\n#=> \"wojtekmach\"\n```\n\nNotice all plugins can be attached to the same request struct which makes it really easy to\nexplore different endpoints.\n\nSee [\"Writing Plugins\" section in `Req.Request` module documentation](https://hexdocs.pm/req/Req.Request.html#module-writing-plugins)\nfor more information.","ref":"changelog.html#plugins"},{"type":"extras","title":"Plug Integration - CHANGELOG","doc":"Req can now be used to easily test plugs using the `:plug` option:\n\n```elixir\ndefmodule Echo do\n  def call(conn, _) do\n    \"/\" <> path = conn.request_path\n    Plug.Conn.send_resp(conn, 200, path)\n  end\nend\n\ntest \"echo\" do\n  assert Req.get!(\"http:///hello\", plug: Echo).body == \"hello\"\nend\n```\n\nyou can define plugs as functions too:\n\n```elixir\ntest \"echo\" do\n  echo = fn conn ->\n    \"/\" <> path = conn.request_path\n    Plug.Conn.send_resp(conn, 200, path)\n  end\n\n  assert Req.get!(\"http:///hello\", plug: echo).body == \"hello\"\nend\n```\n\nwhich is particularly useful to create HTTP service mocks with tools like\n[Bypass](https://github.com/PSPDFKit-labs/bypass).","ref":"changelog.html#plug-integration"},{"type":"extras","title":"Request Adapters - CHANGELOG","doc":"While Req always used Finch as the underlying HTTP client, it was designed from the day one to\neasily swap it out. This is now even easier with an `:adapter` option.\n\nHere is a mock adapter that always returns a successful response:\n\n```elixir\nadapter = fn request ->\n  response = %Req.Response{status: 200, body: \"it works!\"}\n  {request, response}\nend\n\nReq.request!(url: \"http://example\", adapter: adapter).body\n#=> \"it works!\"\n```\n\nHere is another one that uses the [`json/2`][resp_json] function to conveniently\nreturn a JSON response:\n\n[resp_json]: https://hexdocs.pm/req/Req.Response.html#json/2\n\n```elixir\nadapter = fn request ->\n  response = Req.Response.json(%{hello: 42})\n  {request, response}\nend\n\nresp = Req.request!(url: \"http://example\", adapter: adapter)\nresp.headers\n#=> [{\"content-type\", \"application/json\"}]\nresp.body\n#=> %{\"hello\" => 42}\n```\n\nAnd here is a naive Hackney-based adapter and how we can use it:\n\n```elixir\nhackney = fn request ->\n  case :hackney.request(\n         request.method,\n         URI.to_string(request.url),\n         request.headers,\n         request.body,\n         [:with_body]\n       ) do\n    {:ok, status, headers, body} ->\n      headers = for {name, value} <- headers, do: {String.downcase(name), value}\n      response = %Req.Response{status: status, headers: headers, body: body}\n      {request, response}\n\n    {:error, reason} ->\n      {request, RuntimeError.exception(inspect(reason))}\n  end\nend\n\nReq.get!(\"https://api.github.com/repos/elixir-lang/elixir\", adapter: hackney).body[\"description\"]\n#=> \"Elixir is a dynamic, functional language designed for building scalable and maintainable applications\"\n```\n\nSee \"Adapter\" section in `Req.Request` module documentation for more information.","ref":"changelog.html#request-adapters"},{"type":"extras","title":"Major changes - CHANGELOG","doc":"* Add high-level functional API: `Req.new(...) |> Req.request(...)`, `Req.new(...) |>\n    Req.get!(...)`, etc.\n\n  * Add `Req.Request.options` field that steps can read from. Also, make\n    all steps be arity 1.\n\n    When using \"High-level\" API, we now run all steps by default. (The\n    steps, by looking at request.options, can decide to be no-op.)\n\n  * Move low-level API to `Req.Request`\n\n  * Move built-in steps to `Req.Steps`\n\n  * Add step names\n\n  * Add `Req.head!/2`\n\n  * Add `Req.patch!/2`\n\n  * Add `Req.Request.adapter` field\n\n  * Add `Req.Request.merge_options/2`\n\n  * Add `Req.Request.register_options/2`\n\n  * Add `Req.Request.put_header/3`\n\n  * Add `Req.Request.put_headers/2`\n\n  * Add `Req.Request.put_new_header/3`\n\n  * Add `Req.Request.get_header/2`\n\n  * Add `Req.Request.update_private/4`\n\n  * Add `Req.Response.new/1`\n\n  * Add `Req.Response.json/2`\n\n  * Add `Req.Response.get_header/2`\n\n  * Add `Req.Response.put_header/3`\n\n  * Rename `put_if_modified_since` step to `cache`\n\n  * Rename `decompress` step to `decompress_body`\n\n  * Remove `put_default_steps` step\n\n  * Remove `run_steps` step\n\n  * Remove `put_default_headers` step\n\n  * Remove `encode_headers` step. The headers are now encoded in `Req.new/1` and `Req.request/2`\n\n  * Remove `Req.Request.unix_socket` field. Add option on `run_finch` step with the same name\n    instead.\n\n  * Require Elixir 1.12","ref":"changelog.html#major-changes"},{"type":"extras","title":"Step changes - CHANGELOG","doc":"* New step: [`put_plug`]\n\n  * New step: [`put_user_agent`] (replaces part of removed `put_default_headers`)\n\n  * New step: [`compressed`] (replaces part of removed `put_default_headers`)\n\n  * New step: [`compress_body`]\n\n  * New step: [`output`]\n\n  * New step: [`handle_http_errors`]\n\n  * [`put_base_url`]: Ignore base URL if given URL contains scheme\n\n  * [`run_finch`]: Add `:connect_options` which dynamically starts (or re-uses already started)\n    Finch pool with the given connection options.\n\n  * [`run_finch`]: Replace `:finch_options` with `:receive_timeout` and `:pool_timeout` options\n\n  * [`encode_body`]: Add `:form` and `:json` options (previously used as `{:form, data}` and\n    `{:json, data}`)\n\n  * [`cache`]: Include request method in cache key\n\n  * [`decompress_body`], [`compressed`]: Support Brotli\n\n  * [`decompress_body`], [`compressed`]: Support Zstandard\n\n  * [`decode_body`]: Support `decode_body: false` option to disable automatic body decoding\n\n  * [`follow_redirects`]: Change method to GET on 301..303 redirects\n\n  * [`follow_redirects`]: Don't send auth headers on redirect to different scheme/host/port\n    unless `location_trusted: true` is set\n\n  * [`retry`]: The `Retry-After` response header on HTTP 429 responses is now respected\n\n  * [`retry`]: The `:retry` option can now be set to `:safe` (default) to only retry GET/HEAD\n    requests on HTTP 408/429/5xx responses or exceptions, `:always` to always retry, `:never` to never\n    retry, and `fun` - a 1-arity function that accepts either a `Req.Response` or an exception\n    struct and returns boolean whether to retry\n\n  * [`retry`]: The `:retry_delay` option now accepts a function that takes a retry count (starting at 0)\n    and returns the delay. Defaults to a simple exponential backoff: 1s, 2s, 4s, 8s, ...","ref":"changelog.html#step-changes"},{"type":"extras","title":"Deprecations - CHANGELOG","doc":"* Deprecate calling `Req.post!(url, body)` in favour of `Req.post!(url, body: body)`.\n    Also, deprecate `Req.post!(url, {:form, data})` in favour of `Req.post!(url, form: data)`.\n    and `Req.post!(url, {:json, data})` in favour of `Req.post!(url, json: data)`. Same for\n    `Req.put!/2`.\n\n  * Deprecate setting `retry: [delay: delay, max_retries: max_retries]`\n    in favour of `retry_delay: delay, max_retries: max_retries`.\n\n  * Deprecate setting `cache: [dir: dir]` in favour of `cache_dir: dir`.\n\n  * Deprecate Req.build/3 in favour of manually building the struct.","ref":"changelog.html#deprecations"},{"type":"extras","title":"v0.2.2 (2022-04-04) - CHANGELOG","doc":"* Relax Finch version requirement","ref":"changelog.html#v0-2-2-2022-04-04"},{"type":"extras","title":"v0.2.1 (2021-11-24) - CHANGELOG","doc":"* Add `:private` field to Response\n  * Update Finch to 0.9.1","ref":"changelog.html#v0-2-1-2021-11-24"},{"type":"extras","title":"v0.2.0 (2021-11-08) - CHANGELOG","doc":"* Rename `normalize_headers` to `encode_headers`\n  * Rename `prepend_default_steps` to `put_default_steps`\n  * Rename `encode` and `decode` to `encode_body` and `decode_body`\n  * Rename `netrc` to `load_netrc`\n  * Rename `finch` step to `run_finch`\n  * Rename `if_modified_since` to `put_if_modified_since`\n  * Rename `range` to `put_range`\n  * Rename `params` to `put_params`\n  * Rename `request.uri` to `request.url`\n  * Change response/error step contract from `f(req, resp_err)` to `f({req, resp_err})`\n  * Support mime 2.x\n  * Add `Req.Response` struct\n  * Add `put!/3` and `delete!/2`\n  * Add `run_steps/2`\n  * Initial support for UNIX domain sockets\n  * Accept `{module, args}` and `module` as steps\n  * Ensure `get_private` and `put_private` have atom keys\n  * `put_default_steps`: Use MFArgs instead of captures for the default steps\n  * `put_if_modified_since`: Fix generating internet time\n  * `encode_headers`: Encode header values\n  * `retry`: Rename `:max_attempts` to `:max_retries`","ref":"changelog.html#v0-2-0-2021-11-08"},{"type":"extras","title":"v0.1.1 (2021-07-16) - CHANGELOG","doc":"* Fix `append_request_steps/2` and `prepend_request_steps/2` (they did the opposite)\n  * Add `finch/1`","ref":"changelog.html#v0-1-1-2021-07-16"},{"type":"extras","title":"v0.1.0 (2021-07-15) - CHANGELOG","doc":"* Initial release\n\n[`auth`]:                https://hexdocs.pm/req/Req.Steps.html#auth/1\n[`cache`]:               https://hexdocs.pm/req/Req.Steps.html#cache/1\n[`compress_body`]:       https://hexdocs.pm/req/Req.Steps.html#compress_body/1\n[`compressed`]:          https://hexdocs.pm/req/Req.Steps.html#compressed/1\n[`decode_body`]:         https://hexdocs.pm/req/Req.Steps.html#decode_body/1\n[`decompress_body`]:     https://hexdocs.pm/req/Req.Steps.html#decompress_body/1\n[`compress_body`]:       https://hexdocs.pm/req/Req.Steps.html#compress_body/1\n[`encode_body`]:         https://hexdocs.pm/req/Req.Steps.html#encode_body/1\n[`redirect`]:            https://hexdocs.pm/req/Req.Steps.html#redirect/1\n[`handle_http_errors`]:  https://hexdocs.pm/req/Req.Steps.html#handle_http_errors/1\n[`put_base_url`]:        https://hexdocs.pm/req/Req.Steps.html#put_base_url/1\n[`put_params`]:          https://hexdocs.pm/req/Req.Steps.html#put_params/1\n[`put_path_params`]:     https://hexdocs.pm/req/Req.Steps.html#put_path_params/1\n[`put_plug`]:            https://hexdocs.pm/req/Req.Steps.html#put_plug/1\n[`run_plug`]:            https://hexdocs.pm/req/Req.Steps.html#run_plug/1\n[`put_user_agent`]:      https://hexdocs.pm/req/Req.Steps.html#put_user_agent/1\n[`put_range`]:           https://hexdocs.pm/req/Req.Steps.html#put_range/1\n[`retry`]:               https://hexdocs.pm/req/Req.Steps.html#retry/1\n[`run_finch`]:           https://hexdocs.pm/req/Req.Steps.html#run_finch/1\n[`checksum`]:            https://hexdocs.pm/req/Req.Steps.html#checksum/1\n[`verify_checksum`]:     https://hexdocs.pm/req/Req.Steps.html#verify_checksum/1\n[`put_aws_sigv4`]:       https://hexdocs.pm/req/Req.Steps.html#put_aws_sigv4/1\n\n[`Req`]:                        https://hexdocs.pm/req/Req.html\n[`Req.new/1`]:                  https://hexdocs.pm/req/Req.html#new/1\n[`Req.request/2`]:              https://hexdocs.pm/req/Req.html#request/2\n[`Req.run/2`]:                  https://hexdocs.pm/req/Req.html#run/2\n[`Req.run!/2`]:                 https://hexdocs.pm/req/Req.html#run!/2\n[`Req.merge/2`]:                https://hexdocs.pm/req/Req.html#merge/2\n[`Req.parse_message/2`]:        https://hexdocs.pm/req/Req.html#merge/2\n[`Req.cancel_async_request/1`]: https://hexdocs.pm/req/Req.html#merge/2\n\n[`Req.Request`]:                   https://hexdocs.pm/req/Req.Request.html\n[`Req.Request.new/1`]:             https://hexdocs.pm/req/Req.Request.html#new/1\n[`Req.Request.run_request/1`]:     https://hexdocs.pm/req/Req.Request.html#run_request/1\n[`Req.Request.get_option/3`]:      https://hexdocs.pm/req/Req.Request.html#get_option/3\n[`Req.Request.get_option_lazy/2`]: https://hexdocs.pm/req/Req.Request.html#get_option_lazy/2\n[`Req.Request.fetch_option/2`]:    https://hexdocs.pm/req/Req.Request.html#fetch_option/2\n[`Req.Request.fetch_option!/2`]:   https://hexdocs.pm/req/Req.Request.html#fetch_option!/2\n[`Req.Request.delete_option/2`]:   https://hexdocs.pm/req/Req.Request.html#delete_option/2\n[`Req.Request.drop_options/2`]:    https://hexdocs.pm/req/Req.Request.html#drop_options/2\n[`Req.Request.update_private/4`]:  https://hexdocs.pm/req/Req.Request.html#update_private/4\n\n[`Req.Response`]:                  https://hexdocs.pm/req/Req.Response.html\n[`Req.Response.get_header/2`]:     https://hexdocs.pm/req/Req.Response.html#get_response/2\n[`Req.Response.delete_header/2`]:  https://hexdocs.pm/req/Req.Response.html#delete_header/2\n[`Req.Response.update_private/4`]: https://hexdocs.pm/req/Req.Response.html#update_private/4\n[`Req.Response.Async`]:            https://hexdocs.pm/req/Req.Response.Async.html\n\n[`Req.Test`]: https://hexdocs.pm/req/Req.Test.html\n[`Req.Test.stub/2`]: https://hexdocs.pm/req/Req.Test.html#stub/2\n[`Req.Test.json/2`]: https://hexdocs.pm/req/Req.Test.html#json/2\n[`Req.Test.html/2`]: https://hexdocs.pm/req/Req.Test.html#html/2\n[`Req.Test.text/2`]: https://hexdocs.pm/req/Req.Test.html#text/2\n[`Req.Test.allow/3`]: https://hexdocs.pm/req/Req.Test.html#allow/3\n[`Req.Test.transport_error/2`]: https://hexdocs.pm/req/Req.Test.html#transport_error/2\n[`Req.Test.expect/3`]: https://hexdocs.pm/req/Req.Test.html#expect/3\n[`Req.Test.verify!/0`]: https://hexdocs.pm/req/Req.Test.html#verify!/0\n[`Req.Test.verify!/1`]: https://hexdocs.pm/req/Req.Test.html#verify!/1\n[`Req.Test.verify_on_exit!/1`]: https://hexdocs.pm/req/Req.Test.html#verify_on_exit!/1\n[`Req.Test.set_req_test_from_context/1`]: https://hexdocs.pm/req/Req.Test.html#set_req_test_from_context/1\n[`Req.Test.set_req_test_to_private/1`]: https://hexdocs.pm/req/Req.Test.html#set_req_test_to_private/1\n[`Req.Test.set_req_test_to_shared/1`]: https://hexdocs.pm/req/Req.Test.html#set_req_test_to_shared/1\n\n\n[`Req.Steps`]:   https://hexdocs.pm/req/Req.Steps.html\n\n[`Req.TransportError`]: https://hexdocs.pm/req/Req.TransportError.html\n[`Req.HTTPError`]: https://hexdocs.pm/req/Req.HTTPError.html\n[`Req.TooManyRedirectsError`]: https://hexdocs.pm/req/Req.TooManyRedirectsError.html\n[`Req.DecompressError`]: https://hexdocs.pm/req/Req.DecompressError.html\n[`Req.ArchiveError`]: https://hexdocs.pm/req/Req.ArchiveError.html\n\n[`Enumerable`]:  https://hexdocs.pm/elixir/Enumerable.html\n[`Collectable`]: https://hexdocs.pm/elixir/Collectable.html","ref":"changelog.html#v0-1-0-2021-07-15"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}